@@ -6,12 +6,12 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed without
-#  the express permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 import logging
+from typing import Any, Dict
 
 import requests
 
@@ -19,11 +19,9 @@ LOGGER = logging.getLogger(__name__)
 DEFAULT_JUPYTER_INTERACTIVE_FILE_NAME = "Jupyter interactive"
 DEFAULT_JUPYTER_CODE_ASSET_NAME = "Code.ipynb"
 DEFAULT_COLAB_NOTEBOOK_ASSET_NAME = "ColabNotebook.ipynb"
-COLAB_SESSION_URL = "http://172.28.0.2:9000/api/sessions"
 
 
-def _in_jupyter_environment():
-    # type: () -> bool
+def _in_jupyter_environment() -> bool:
     """
     Check to see if code is running in a Jupyter environment,
     including jupyter notebook, lab, or console.
@@ -40,8 +38,7 @@ def _in_jupyter_environment():
         return True
 
 
-def _in_ipython_environment():
-    # type: () -> bool
+def _in_ipython_environment() -> bool:
     """
     Check to see if code is running in an IPython environment.
     """
@@ -57,8 +54,7 @@ def _in_ipython_environment():
         return True
 
 
-def _in_colab_environment():
-    # type: () -> bool
+def _in_colab_environment() -> bool:
     """
     Check to see if code is running in Google colab.
     """
@@ -71,33 +67,47 @@ def _in_colab_environment():
     return "google.colab" in str(ipy)
 
 
-def _get_notebook_url():
-    try:
-        notebook_file_id = requests.get(COLAB_SESSION_URL, timeout=1).json()[0]["path"]
-        return "https://colab.research.google.com/notebook#%s" % notebook_file_id
-    except Exception:
-        LOGGER.debug("Failed to retrieve the fileId of the notebook", exc_info=True)
-        return None
+def _get_colab_session_api_url() -> str:
+    """For manual IP check use the following inside Colab notebook:
+        # Gives IP address
+        !hostname -I
+        print()  # empty line
+        # Gives IP addresses with port numbers
+        !sudo lsof -i -P -n | grep LISTEN
+    OR
+        colab_ip = %system hostname -I   # uses colab magic to get list from bash
+        colab_ip = colab_ip[0].strip()   # returns "172.28.0.12"
+        colab_port = 9000                # could use 6000, 8080, or 9000
+
+    https://stackoverflow.com/questions/61901628/it-is-possible-to-assign-the-google-colab-notebook-name-to-a-python-variable-in
+    """
+    from socket import gethostbyname, gethostname
+
+    ip = gethostbyname(gethostname())
+    return f"http://{ip}:9000/api/sessions"  # could use any of the ports: 6000, 8080, or 9000
 
 
-def _get_colab_notebook_json():
-    try:
-        import google.colab._message
-
-        notebook_json = google.colab._message.blocking_request(request_type="get_ipynb")
-    except Exception:
-        LOGGER.debug("Failed to get Google Colab notebook content", exc_info=True)
-        return None
-
-    try:
-        return notebook_json["ipynb"]
-    except Exception:
-        LOGGER.debug("Failed to parse Google Colab payload", exc_info=True)
-        return None
+def _get_colab_notebook_url() -> str:
+    response = requests.get(_get_colab_session_api_url(), timeout=1)
+    response.raise_for_status()
+    notebook_file_id = response.json()[0]["path"]
+    return "https://colab.research.google.com/notebook#%s" % notebook_file_id
 
 
-def display_or_open_browser(url, clear=False, wait=True, new=0, autoraise=True):
-    # type: (str, bool, bool, int, bool) -> None
+def _get_colab_notebook_json() -> Dict[str, Any]:
+    import google.colab._message
+
+    notebook_json = google.colab._message.blocking_request(request_type="get_ipynb")
+    return notebook_json["ipynb"]
+
+
+def display_or_open_browser(
+    url: str,
+    clear: bool = False,
+    wait: bool = True,
+    new: int = 0,
+    autoraise: bool = True,
+) -> None:
     if _in_jupyter_environment():
         from IPython.display import IFrame, clear_output, display
 
