@@ -6,13 +6,12 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed
-#  without the express permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 
-import imp
+import importlib.machinery
 import os.path
 import sys
 
@@ -32,9 +31,10 @@ if __name__ == "sitecustomize":
 
     # Then import any other sitecustomize
     try:
-        _file, pathname, description = imp.find_module("sitecustomize", path)
+        module_spec = importlib.machinery.PathFinder().find_spec("sitecustomize", path)
     except ImportError:
         # We might be the only sitecustomize file
         pass
     else:
-        imp.load_module("sitecustomize", _file, pathname, description)
+        if module_spec is not None:
+            module_spec.loader.load_module("sitecustomize")
