@@ -6,15 +6,13 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
-#  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed without
-#  the express permission of Comet ML Inc.
+#  Sign up for free at https://www.comet.com
+#  Copyright (C) 2015-2024 Comet ML INC
+#  This source code is licensed under the MIT license.
 # *******************************************************
 
 import logging
-
-import six
+from typing import Callable
 
 from .._typing import (
     Any,
@@ -28,8 +26,9 @@ from .._typing import (
     Tuple,
 )
 from ..data_structure import Histogram
-from ..experiment import BaseExperiment
+from ..experiment import CometExperiment
 from ..logging_messages import TF_KERAS_CALLBACK_WARNING_CLOSED_EXPERIMENT
+from ..messages import ParameterMessage
 from ..utils import get_time_monotonic, tensor_length
 
 LOGGER = logging.getLogger(__name__)
@@ -38,8 +37,9 @@ XGBOOST_PARAMS_EXCLUDE = {"model", "evaluation_result_list", "iteration", "handl
 XGBOOST_BOOSTER_ATTRS_EXCLUDE = {"handle"}
 
 
-def _filter_xgboost_attributes(obj, exclude_set):
-    # type: (Any, Set[int]) -> Generator[Tuple[str, Any], None, None]
+def _filter_xgboost_attributes(
+    obj: Any, exclude_set: Set[str]
+) -> Generator[Tuple[str, Any], None, None]:
     for attribute_name in dir(obj):
         if attribute_name.startswith("_"):
             continue
@@ -47,7 +47,11 @@ def _filter_xgboost_attributes(obj, exclude_set):
         if attribute_name in exclude_set:
             continue
 
-        attribute_value = getattr(obj, attribute_name)
+        try:
+            attribute_value = getattr(obj, attribute_name)
+        except AttributeError:
+            LOGGER.debug("Failed to log the XGBoost metrics", exc_info=True)
+            continue
 
         if callable(attribute_value):
             continue
@@ -55,17 +59,17 @@ def _filter_xgboost_attributes(obj, exclude_set):
         if attribute_value is None:
             continue
 
-        yield (attribute_name, attribute_value)
+        yield attribute_name, attribute_value
 
 
-def _get_xgboost_env_params(env):
-    # type: (Any) -> Generator[Tuple[str, Any], None, None]
+def _get_xgboost_env_params(env: Any) -> Generator[Tuple[str, Any], None, None]:
     for attribute in _filter_xgboost_attributes(env, XGBOOST_PARAMS_EXCLUDE):
         yield attribute
 
 
-def _get_xgboost_env_model_params(booster):
-    # type: (Any) -> Generator[Tuple[str, Any], None, None]
+def _get_xgboost_env_model_params(
+    booster: Any,
+) -> Generator[Tuple[str, Any], None, None]:
     for attribute in _filter_xgboost_attributes(booster, XGBOOST_BOOSTER_ATTRS_EXCLUDE):
         yield attribute
 
@@ -83,18 +87,18 @@ def _get_xgboost_booster_graph(booster):
 
 
 def _log_xgboost_step(experiment, xgboost_step):
-    # type: (BaseExperiment, int) -> None
+    # type: (CometExperiment, int) -> None
     """
     Logs the step of the training to the experiment
     """
     try:
-        experiment.set_step(xgboost_step)
+        experiment._set_step(xgboost_step)
     except Exception:
         LOGGER.debug("Failed to log the XGBoost step", exc_info=True)
 
 
 def _log_xgboost_parameters(experiment, obj):
-    # type: (BaseExperiment, Any) -> None
+    # type: (CometExperiment, Any) -> None
     """
     Logs the parameters of the training to the experiment
     """
@@ -103,7 +107,10 @@ def _log_xgboost_parameters(experiment, obj):
             if not experiment._storage["xgboost"]["env_parameter_set"]:
                 for attribute_name, attribute_value in _get_xgboost_env_params(obj):
                     experiment._log_parameter(
-                        attribute_name, attribute_value, framework="xgboost"
+                        attribute_name,
+                        attribute_value,
+                        framework="xgboost",
+                        source=ParameterMessage.source_autologger,
                     )
 
                 # Set only once the parameters
@@ -113,7 +120,7 @@ def _log_xgboost_parameters(experiment, obj):
 
 
 def _log_xgboost_model_attributes(experiment, booster):
-    # type: (BaseExperiment, Any) -> None
+    # type: (CometExperiment, Any) -> None
     """
     Logs the attributes of the model to the experiment
     """
@@ -125,7 +132,10 @@ def _log_xgboost_model_attributes(experiment, booster):
                     booster
                 ):
                     experiment._log_parameter(
-                        attribute_name, attribute_value, framework="xgboost"
+                        attribute_name,
+                        attribute_value,
+                        framework="xgboost",
+                        source=ParameterMessage.source_autologger,
                     )
 
                 # Set only once the model parameters
@@ -135,7 +145,7 @@ def _log_xgboost_model_attributes(experiment, booster):
 
 
 def _log_xgboost_model_metrics(experiment, evals_log):
-    # type: (BaseExperiment, Dict) -> None
+    # type: (CometExperiment, Dict) -> None
     """
     Logs the metrics to the experiment
     """
@@ -150,7 +160,7 @@ def _log_xgboost_model_metrics(experiment, evals_log):
 
 
 def _log_xgboost_model_graph(experiment, booster):
-    # type: (BaseExperiment, Any) -> None
+    # type: (CometExperiment, Any) -> None
     """
     Logs the graph of the model to the experiment
     """
@@ -353,7 +363,7 @@ def get_tensorflow_gradient_histograms(
         all_weight_names = [weight.name for weight in weights]
         # For each index:
         index = 0
-        for ins, targs in six.moves.zip(input_gen, target_gen):
+        for ins, targs in zip(input_gen, target_gen):
             gradients = get_gradients(model, ins, targs, weights)
 
             if histograms is None:
@@ -376,7 +386,7 @@ def get_tensorflow_gradient_histograms(
         names = [weight.name for weight in weights]
         indices = ["all" for weight in weights]  # type: List[TensorflowInput]
 
-        for batch_input, batch_target in six.moves.zip(input_gen, target_gen):
+        for batch_input, batch_target in zip(input_gen, target_gen):
             gradients = get_gradients(model, batch_input, batch_target, weights)
 
             if histograms is None:
@@ -386,23 +396,34 @@ def get_tensorflow_gradient_histograms(
                 histograms[i].add(gradients[i])
 
     # check to see if all are the same length
-    return six.moves.zip(names, indices, histograms)
+    return zip(names, indices, histograms)
 
 
 def get_tensorflow_activation_histogram_indices(
-    model, output_tensor, inputs, index_list
-):
-    # type: (Any, Any, Any, List[int]) -> Optional[List[Histogram]]
-    # Logging activations does not work with tensorflow 1.11
+    model: Any, output_tensor: Any, inputs: Any, index_list: List[int]
+) -> Optional[List[Histogram]]:
+    """
+    Compute activation histograms for specific indices of a TensorFlow/Keras model.
+
+    Args:
+        model: A TensorFlow/Keras Model instance (e.g., tf.keras.Model).
+        output_tensor: A TensorFlow tensor representing the model's output or a specific layer's output.
+        inputs: Input data to the model (e.g., numpy array, TensorFlow tensor, or list thereof).
+        index_list: List of indices specifying which samples to compute histograms for.
+    Returns:
+        A list of Histogram objects for each index in index_list, or None if an error occurs.
+    """
     try:
         import tensorflow as tf
     except ImportError:
         return None
 
-    histograms = [Histogram() for i in range(len(index_list))]
-
     try:
-        function = tf.keras.backend.function(model.inputs, output_tensor)
+        function = _get_keras_function(model, output_tensor)
+        if function is None:
+            return None
+
+        histograms = [Histogram() for _ in range(len(index_list))]
 
         for i, batch_input in enumerate_tensor_list(
             len(model.inputs), inputs, index_list
@@ -424,18 +445,21 @@ def get_tensorflow_activation_histogram_indices(
         return None
 
 
-def get_tensorflow_activation_histogram_all(model, output_tensor, inputs, batch_size):
-    # type: (Any, Any, Any, int) -> Optional[List[Histogram]]
-    # Logging activations does not work with tensorflow 1.11
+def get_tensorflow_activation_histogram_all(
+    model: Any, output_tensor: Any, inputs: Any, batch_size: int
+) -> Optional[List[Histogram]]:
+
     try:
         import tensorflow as tf
     except ImportError:
         return None
 
-    histogram = Histogram()
-
     try:
-        function = tf.keras.backend.function(model.inputs, output_tensor)
+        function = _get_keras_function(model, output_tensor)
+        if function is None:
+            return None
+
+        histogram = Histogram()
 
         for batch_input in enumerate_tensors(len(model.inputs), inputs, batch_size):
             # Batch input is either a one-item input (tensor/ndarray) or a list of one-item
@@ -519,17 +543,18 @@ def build_empty_keras_callback(base):
 
 def build_keras_callback(base):
     class CometKerasCallback(base):  # type: ignore
-        """Keras callback to report params, metrics to Comet.ml Experiment()"""
+        """Keras callback to report params, metrics to Comet"""
 
         def __init__(
             self,
-            experiment,  # type: BaseExperiment
-            log_params=None,  # type: Optional[bool]
-            log_metrics=None,  # type: Optional[bool]
-            log_graph=None,  # type: Optional[bool]
-            log_histograms=None,  # type: Optional[bool]
-            inputs=None,  # type: Optional[Any]
-            targets=None,  # type: Optional[Any]
+            experiment: CometExperiment,
+            log_params: Optional[bool] = None,
+            log_metrics: Optional[bool] = None,
+            log_graph: Optional[bool] = None,
+            log_histograms: Optional[bool] = None,
+            inputs: Optional[Any] = None,
+            targets: Optional[Any] = None,
+            initial_validation_step: int = 0,
         ):
             # type: (...) -> None
             """
@@ -565,9 +590,13 @@ def build_keras_callback(base):
             # Inits the experiment with reference to the name of this class. Required for loading the correct
             # script file
             self.experiment = experiment
+
+            if self.experiment.curr_step is None:
+                self.experiment.curr_step = 0
+
+            self._test_step = initial_validation_step
+
             self.epoch_start_time = None  # type: Optional[float]
-            self.our_step = 0
-            self.our_epoch = 0
 
             self.activation_ignore_list = ["flatten", "dropout", "activation"]
 
@@ -580,10 +609,10 @@ def build_keras_callback(base):
             try:
                 # This function should only be called during train mode.
                 LOGGER.debug("On epoch begin %s %s", epoch, logs)
-                self.experiment.set_epoch(self.our_epoch)
+                self.experiment._set_epoch(epoch)
                 self.epoch_start_time = get_time_monotonic()
 
-                if self.our_epoch == 0:
+                if epoch == 0:
                     self._log_histograms()
             except Exception:
                 LOGGER.warning(
@@ -605,19 +634,19 @@ def build_keras_callback(base):
                         self.experiment._log_metric(
                             "epoch_duration",
                             get_time_monotonic() - self.epoch_start_time,
-                            step=self.our_step,
-                            epoch=self.our_epoch,
+                            step=self.experiment.curr_step,
+                            epoch=epoch,
                             framework="keras",
                         )
                         self.epoch_start_time = None
-                    self.experiment.log_epoch_end(self.our_epoch, step=self.our_step)
+                    self.experiment.log_epoch_end(epoch, step=self.experiment.curr_step)
                     if logs:
                         for name, val in logs.items():
                             self.experiment._log_metric(
                                 name,
                                 val,
-                                step=self.our_step,
-                                epoch=self.our_epoch,
+                                step=self.experiment.curr_step,
+                                epoch=epoch,
                                 framework="keras",
                             )
             except Exception:
@@ -626,10 +655,12 @@ def build_keras_callback(base):
                     exc_info=True,
                 )
 
-            self.our_epoch += 1
+            self.experiment.curr_epoch = epoch + 1
 
             try:
-                if self.experiment._check_histogram_epoch_report_rate(self.our_epoch):
+                if self.experiment._check_histogram_epoch_report_rate(
+                    self.experiment.curr_epoch
+                ):
                     self._log_histograms()
             except Exception:
                 LOGGER.warning(
@@ -684,8 +715,8 @@ def build_keras_callback(base):
                             self.experiment.log_histogram_3d(
                                 weights[i],
                                 name=layer.weights[i].name,
-                                step=self.our_step,
-                                epoch=self.our_epoch,
+                                step=self.experiment.curr_step,
+                                epoch=self.experiment.curr_epoch,
                                 metadata={"prefix": prefix},
                             )
                     else:
@@ -758,8 +789,8 @@ def build_keras_callback(base):
                         self.experiment.log_histogram_3d(
                             histogram,
                             name="/".join([layer_name, ("gradients:%s" % index)]),
-                            step=self.our_step,
-                            epoch=self.our_epoch,
+                            step=self.experiment.curr_step,
+                            epoch=self.experiment.curr_epoch,
                             metadata={"prefix": prefix},
                         )
             except Exception:
@@ -852,8 +883,8 @@ def build_keras_callback(base):
                                 name="/".join(
                                     [layer.name, ("activations:%s" % name)]
                                 ),  # index of input tensor
-                                step=self.our_step,
-                                epoch=self.our_epoch,
+                                step=self.experiment.curr_step,
+                                epoch=self.experiment.curr_epoch,
                                 metadata={"prefix": prefix},
                             )
 
@@ -888,7 +919,7 @@ def build_keras_callback(base):
                 # This function called directly when in train mode.
                 LOGGER.debug("On batch end %s %s", batch, logs)
 
-                self.our_step += 1
+                self.experiment.curr_step += 1
 
                 # Use the batch from keras, as it starts over each epoch:
                 if self.experiment._check_metric_step_report_rate(batch):
@@ -912,19 +943,36 @@ def build_keras_callback(base):
 
         def on_test_batch_end(self, batch, logs=None):
             try:
-                with self.experiment.validate():
-                    self.on_batch_end(batch, logs)
+                if self.experiment.context is not None:
+                    # append existing context to support multiple fit calls
+                    context = "%s_%s" % (self.experiment.context, "validate")
+                else:
+                    context = "validate"
+                with self.experiment.context_manager(context):
+                    self._on_batch_end_with_test_step(batch, logs)
             except Exception:
                 LOGGER.warning(
                     "An unknown exception happened in Keras callback on_test_batch_end; ignoring",
                     exc_info=True,
                 )
 
+        def _on_batch_end_with_test_step(self, batch, logs=None):
+            train_step = self.experiment.curr_step
+            try:
+                self.experiment.curr_step = self._test_step
+                self.on_batch_end(batch, logs)
+            finally:
+                self._test_step = self.experiment.curr_step
+                self.experiment.curr_step = train_step
+
         def _send_batch_messages(self, logs):
             if logs and self.experiment.auto_metric_logging:
                 for name, val in logs.items():
                     self.experiment._log_metric(
-                        "batch_" + name, val, step=self.our_step, framework="keras"
+                        "batch_" + name,
+                        val,
+                        step=self.experiment.curr_step,
+                        framework="keras",
                     )
 
         def on_train_begin(self, logs=None):
@@ -958,20 +1006,33 @@ def build_keras_callback(base):
                 if self.experiment.auto_param_logging:
                     if logs:
                         for k, v in logs.items():
-                            self.experiment._log_parameter(k, v, framework="keras")
+                            self.experiment._log_parameter(
+                                k,
+                                v,
+                                framework="keras",
+                                source=ParameterMessage.source_autologger,
+                            )
 
                     # Keras Callback doesn't set this parameter at creation by default
                     if hasattr(self, "params") and self.params:
                         for k, v in self.params.items():
                             if k != "metrics":
-                                self.experiment._log_parameter(k, v, framework="keras")
+                                self.experiment._log_parameter(
+                                    k,
+                                    v,
+                                    framework="keras",
+                                    source=ParameterMessage.source_autologger,
+                                )
 
                     try:
                         optimizer_name = self.model.optimizer.__class__.__name__
                         config = self.model.optimizer.get_config()
                         for key, value in config.items():
                             self.experiment._log_parameter(
-                                optimizer_name + "_" + key, value, framework="keras"
+                                optimizer_name + "_" + key,
+                                value,
+                                framework="keras",
+                                source=ParameterMessage.source_autologger,
                             )
                     except Exception:
                         LOGGER.debug(
@@ -1001,7 +1062,7 @@ def build_keras_callback(base):
 
 
 def get_keras_model(experiment, model):
-    # type: (BaseExperiment, Any) -> Any
+    # type: (CometExperiment, Any) -> Any
 
     # With multi-gpu models we save the original model in the experiment
     # storage
@@ -1015,12 +1076,13 @@ def get_keras_model(experiment, model):
 
 
 def check_experiment_closed_and_warn(
-    experiment, show_warning=True, callback_method_name="undefined"
-):
-    # type: (BaseExperiment, Optional[bool], Optional[str]) -> bool
+    experiment: CometExperiment,
+    show_warning: bool = True,
+    callback_method_name: str = "undefined",
+) -> bool:
     """Checks if specified experiment is closed and display warning
     Args:
-        experiment: BaseExperiment - the experiment to be checked
+        experiment: CometExperiment - the experiment to be checked
         show_warning: Optional. bool - the flag to indicate if warning should be displayed [default: True]
         callback_method_name: Optional. string - the name of callback method related to this check.
     """
@@ -1030,3 +1092,29 @@ def check_experiment_closed_and_warn(
         )
 
     return not experiment.alive
+
+
+def _get_keras_function(model: Any, output_tensor: Any) -> Optional[Callable]:
+    try:
+        import keras
+
+        # try Keras 2.x API
+        if hasattr(keras.backend, "function"):
+            return keras.backend.function(model.inputs, output_tensor)
+
+        # try Keras 3.x API
+        from keras.models import Model
+
+        return Model(inputs=model.inputs, outputs=output_tensor)
+    except ImportError:
+        pass
+
+    try:
+        import tensorflow
+
+        # support for older versions of Keras API
+        return tensorflow.keras.backend.function(model.inputs, output_tensor)
+    except ImportError:
+        pass
+
+    return None
