@@ -6,24 +6,23 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed without
-#  the express permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 
 import json
 import logging
 
 from .._typing import Any
-from ..experiment import BaseExperiment
+from ..experiment import CometExperiment
+from ..messages import ParameterMessage
 
 LOGGER = logging.getLogger(__name__)
 
 
 class CometLGBMCallback(object):
-    def __init__(self, experiment):
-        # type: (BaseExperiment) -> None
+    def __init__(self, experiment: CometExperiment) -> None:
         """
         Make and return a lightgbm callback for training.
         """
@@ -52,7 +51,12 @@ class CometLGBMCallback(object):
                     )
 
             if self.experiment.auto_param_logging:
-                self.experiment._log_parameters(env.params, framework="lightgbm")
+                self.experiment._log_parameters(
+                    env.params,
+                    framework="lightgbm",
+                    source=ParameterMessage.source_autologger,
+                    flatten_nested=False,
+                )
 
             self.initialized = True
 
