@@ -6,10 +6,9 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed without
-#  the express permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 from __future__ import print_function
 
@@ -27,12 +26,23 @@ from .convert_utils import (
     numpy,
 )
 from .logging_messages import (
+    CONFUSION_MATRIX_COMPUTE_MATRIX_INVALID_Y_PREDICTOR_WARNING,
+    CONFUSION_MATRIX_COMPUTE_MATRIX_INVALID_Y_TRUE_WARNING,
+    CONFUSION_MATRIX_CONVERT_TO_MATRIX_FAILED_ERROR,
     CONFUSION_MATRIX_ERROR_RESULTING_LENGTH,
     CONFUSION_MATRIX_ERROR_WRONG_LENGTH,
     CONFUSION_MATRIX_EXAMPLE_DICT_INVALID_FORMAT,
     CONFUSION_MATRIX_EXAMPLE_INVALID_TYPE,
     CONFUSION_MATRIX_EXAMPLE_NONE,
+    CONFUSION_MATRIX_IMAGE_HASH_FAILED_INFO,
+    CONFUSION_MATRIX_IMAGE_INDEX_EXPERIMENT_NOT_SET_INFO,
+    CONFUSION_MATRIX_IMAGE_INDEX_FAILED_TO_GENERATE_IMAGE_INFO,
+    CONFUSION_MATRIX_IMAGE_INDEX_IMAGES_NOT_SET_INFO,
     CONFUSION_MATRIX_INDEX_TO_EXAMPLE_ERROR,
+    CONFUSION_MATRIX_NUM_LABELS_NOT_EQUAL_CATEGORIES_EXCEPTION,
+    CONFUSION_MATRIX_SHOULD_BE_TWO_DIMENSIONAL_EXCEPTION,
+    CONFUSION_MATRIX_TRUE_AND_PREDICTED_REQUIRED_EXCEPTION,
+    CONFUSION_MATRIX_TRUE_PREDICTED_OR_MATRIX_EXCEPTION,
 )
 from .utils import is_list_like, log_once_at_level
 
@@ -40,8 +50,7 @@ LOGGER = logging.getLogger(__name__)
 LOG_ONCE_CACHE = set()  # type: Set[str]
 
 
-def convert_to_matrix(matrix, dtype=None):
-    # type: (Any, Optional[type]) -> List
+def convert_to_matrix(matrix: Any, dtype: Optional[type] = None) -> List:
     """
     Convert an unknown item into a list of lists of scalars
     and ensure type is dtype (if given).
@@ -55,15 +64,14 @@ def convert_to_matrix(matrix, dtype=None):
     # Next, convert to lists of scalars:
     if hasattr(matrix, "tolist"):  # numpy array
         if len(matrix.shape) != 2:
-            raise ValueError("matrix should be two dimensional")
+            raise ValueError(CONFUSION_MATRIX_SHOULD_BE_TWO_DIMENSIONAL_EXCEPTION)
         return matrix.tolist()
     else:
         # assume it is something we can iterate over:
         return [convert_to_list(row, dtype=dtype) for row in matrix]
 
 
-def convert_to_matrix_dict(matrix):
-    # type: (Any) -> Dict
+def convert_to_matrix_dict(matrix: Any) -> Dict:
     """
     Convert a matrix into a sparse representation using
     dict[(x,y)] = value where value > 0.
@@ -99,7 +107,9 @@ class ConfusionMatrix(object):
         cache=True,
         selected=None,
         images=None,
+        images_annotations=None,
         experiment=None,
+        examples=None,
         **kwargs  # keyword args for index_to_example_function
     ):
         """
@@ -113,13 +123,27 @@ class ConfusionMatrix(object):
                 values, or a list of integers representing the output. If
                 not provided, then matrix may be provided.
             images: (optional) a list of data that can be passed to
-                Experiment.log_image()
+                CometExperiment.log_image()
+            images_annotations: (optional) Specifies annotation layers for images.
+                This parameter expects a list of lists, where each inner list defines an
+                annotation layer, potentially containing bounding boxes and/or polygons.
+
+                For a comprehensive understanding of how annotations are utilized and structured,
+                please consult the documentation for `Experiment.log_image()`.
+
+                **Important Note for Custom `index_to_example_function` Implementations:**
+                If you provide your own implementation for `index_to_example_function` (bypassing the default),
+                it is critical to ensure that every `images_annotations` entry is consistently passed alongside its corresponding image within each `log_image()` call.
+                Furthermore, the metadata associated with these annotations must precisely align with the expectations and requirements of the `Experiment.log_image()` call.
             labels: (optional) a list of strings that name of the
                 columns and rows, in order. By default, it will be
                 "0" through the number of categories (e.g., rows/columns).
             matrix: (optional) the confusion matrix (list of lists).
                 Must be square, if given. If not given, then it is
                 possible to provide y_true and y_predicted.
+            examples: (optional) if matrix is given you may also give
+                examples, a dictionary of {(x, y): [index1, index2, ...]}
+                where x is column and y is row of matrix
             title: (optional) a custom name to be displayed. By
                 default, it is "Confusion Matrix".
             row_label: (optional) label for rows. By default, it is
@@ -148,14 +172,14 @@ class ConfusionMatrix(object):
                 then selected will be computed automatically by selecting
                 the most confused categories.
             experiment: (optional) passed automatically when using
-                `Experiment.create_confusion_matrix()`.
+                [comet_ml.CometExperiment.create_confusion_matrix][].
             kwargs: (optional) any extra keywords and their values will
                 be passed onto the index_to_example_function.
 
         Note:
             The matrix is [row][col] and [real][predicted] order. That way, the
             data appears as it is display in the confusion matrix in the user
-            interface on comet.ml.
+            interface on comet.com.
 
         Note:
             Uses winner_function to compute winning categories for
@@ -164,9 +188,10 @@ class ConfusionMatrix(object):
         Example:
 
         ```python
+        >>> import comet_ml
         # Typically, you can log the y_true/y_predicted or matrix:
 
-        >>> experiment = Experiment()
+        >>> experiment = comet_ml.start()
 
         # If you have a y_true and y_predicted:
         >>> y_predicted = model.predict(x_test)
@@ -178,16 +203,16 @@ class ConfusionMatrix(object):
                                             [2, 2, 2, 2]) # guess 2 for all
 
         # Or, if you have already computed the matrix:
-        >>> experiment.log_confusion_matrix(labels=["one", two", three"],
-                                            matrix=[[10, 0, 0]
-                                                    [ 0, 9, 1]
-                                                    [ 1, 1, 8]])
+        >>> experiment.log_confusion_matrix(labels=["one", "two", "three"],
+                                            matrix=[[10, 0, 0],
+                                                    [ 0, 9, 1],
+                                                    [ 1, 1, 8],])
 
         # However, if you want to reuse examples from previous runs,
         # you can reuse a ConfusionMatrix instance. You might want to
         # do this if you are creating a series of confusion matrices
         # during the training of a model.
-        # See https://staging.comet.ml/docs/quick-start/ for a tutorial.
+        # See https://staging.comet.com/docs/quick-start/ for a tutorial.
 
         >>> cm = ConfusionMatrix()
         >>> y_predicted = model.predict(x_test)
@@ -202,13 +227,14 @@ class ConfusionMatrix(object):
 
         For more details and example uses, please see:
 
-        https://www.comet.ml/docs/python-sdk/Comet-Confusion-Matrix/
+        [Log Confusion Matrix](/docs/v2/guides/experiment-management/log-data/confusion-matrix/)
 
         or:
 
-        https://www.comet.ml/docs/python-sdk/Experiment/#experimentlog_confusion_matrix
+        [comet_ml.CometExperiment.log_confusion_matrix][]
         """
         self.images = images
+        self.images_annotations = images_annotations
         self.experiment = experiment
         if self.images is not None:
             if index_to_example_function is None:
@@ -216,14 +242,12 @@ class ConfusionMatrix(object):
 
         if y_true is not None and y_predicted is not None:
             if matrix is not None:
-                raise ValueError(
-                    "you need to give either (y_true and y_predicted) or matrix, NOT both"
-                )
+                raise ValueError(CONFUSION_MATRIX_TRUE_PREDICTED_OR_MATRIX_EXCEPTION)
             # else fine
         elif y_true is None and y_predicted is None:
             pass  # fine
         elif y_true is None or y_predicted is None:
-            raise ValueError("if you give y_true OR y_predicted you must give both")
+            raise ValueError(CONFUSION_MATRIX_TRUE_AND_PREDICTED_REQUIRED_EXCEPTION)
 
         if winner_function is not None:
             self.winner_function = winner_function
@@ -245,20 +269,29 @@ class ConfusionMatrix(object):
         self.use_cache = cache
         self.clear_cache()
         self.clear()
-        self.images = None
         self._need_init = True
         self._example_matrix = {}
         self._dimension = None  # type: Optional[int]
 
         if y_true is not None and y_predicted is not None:
-            self.compute_matrix(y_true, y_predicted, images=images, **kwargs)
+            self.compute_matrix(
+                y_true,
+                y_predicted,
+                images=images,
+                images_annotations=images_annotations,
+                **kwargs
+            )
         elif matrix is not None:
+            if self._need_init:
+                self.initialize()
             try:
                 self._matrix = convert_to_matrix_dict(matrix)
                 self._dimension = len(matrix)
+                if examples:
+                    self._process_examples(examples, **kwargs)
             except Exception:
                 LOGGER.error(
-                    "convert_to_matrix failed; confusion matrix not generated",
+                    CONFUSION_MATRIX_CONVERT_TO_MATRIX_FAILED_ERROR,
                     exc_info=True,
                 )
 
@@ -267,8 +300,8 @@ class ConfusionMatrix(object):
         Method to call when you want to reset the confusion matrix.
         Doesn't reset cache.
 
-        Note: this method is called when the confusion matrix
-            is logged.
+        Note:
+            This method is called automatically when the confusion matrix is logged.
         """
         self._need_init = True
 
@@ -282,6 +315,7 @@ class ConfusionMatrix(object):
         self._matrix = None
         self._dimension = None  # type: Optional[int]
         self.images = None
+        self.images_annotations = None
 
     def clear_cache(self):
         """
@@ -306,7 +340,7 @@ class ConfusionMatrix(object):
         of patterns to apply winner function to.
 
         Args:
-            ndarry: a 2-D matrix where rows are the patterns
+            ndarray: a 2-D matrix where rows are the patterns
 
         Returns a list of winning categories.
         """
@@ -339,10 +373,10 @@ class ConfusionMatrix(object):
             kwargs: additional keyword arguments for an overridden method
 
         Returns:
-            * an integer representing the winning cateory
-            * a string representing a example
-            * a string representing a URL (starts with "http")
-            * a dictionary containing keys "sample" and "assetId"
+            an integer representing the winning category
+            a string representing an example
+            a string representing an URL (starts with "http")
+            a dictionary containing keys "sample" and "assetId"
 
         The return dictionary is used to link a confusion matrix cell
         with a Comet asset. In this function, you can create an asset
@@ -367,36 +401,45 @@ class ConfusionMatrix(object):
         """
         return index
 
-    def image_index_to_example_function(self, index, **kwargs):
-        # type: (int, Dict[str, Any]) -> Optional[Dict[str, Any]]
+    def image_index_to_example_function(
+        self, index: int, **kwargs: Dict[str, Any]
+    ) -> Optional[Dict[str, Any]]:
         """
         The internal index_to_example function used when
         passing in images to the compute_matrix() method.
 
         Args:
             index: the index of the pattern being tested
-            kwargs: additional keyword arguments for Experiment.log_image()
+            kwargs: additional keyword arguments for [comet_ml.CometExperiment.log_image]
 
         Returns:
-            * a dictionary containing keys "sample" and "assetId"
+            a dictionary containing keys "sample" and "assetId"
         """
         if self.images is None:
-            log_once_at_level(logging.INFO, "images were not set; ignoring examples")
+            log_once_at_level(
+                logging.INFO, CONFUSION_MATRIX_IMAGE_INDEX_IMAGES_NOT_SET_INFO
+            )
             return None
 
         if self.experiment is None:
             log_once_at_level(
                 logging.INFO,
-                "experiment is not set; use experiment.create_confusion_matrix(); ignoring examples",
+                CONFUSION_MATRIX_IMAGE_INDEX_EXPERIMENT_NOT_SET_INFO,
             )
             return None
 
         image_array = self.images[index]
+        image_annotations = (
+            self.images_annotations[index] if self.images_annotations else None
+        )
+
         image_name = "confusion-matrix-%05d.png" % index
-        result = self.experiment.log_image(image_array, name=image_name, **kwargs)
+        result = self.experiment.log_image(
+            image_array, name=image_name, annotations=image_annotations, **kwargs
+        )
         if result is None:
             log_once_at_level(
-                logging.INFO, "unable to generate image from images; ignoring example"
+                logging.INFO, CONFUSION_MATRIX_IMAGE_INDEX_FAILED_TO_GENERATE_IMAGE_INFO
             )
             return None
 
@@ -529,7 +572,7 @@ class ConfusionMatrix(object):
                 return hash(array_bytes)
             except Exception:
                 log_once_at_level(
-                    logging.INFO, "unable to generate hash from image", exc_info=True
+                    logging.INFO, CONFUSION_MATRIX_IMAGE_HASH_FAILED_INFO, exc_info=True
                 )
                 return None
         else:
@@ -566,8 +609,9 @@ class ConfusionMatrix(object):
         key = self._get_cache_key(index)
         self._cache_example[key] = example
 
-    def _example_from_list(self, indices, x, y, **kwargs):
-        # type: (Set[int], int, int, Any) -> Set[int]
+    def _example_from_list(
+        self, indices: Set[int], x: int, y: int, **kwargs: Any
+    ) -> Set[int]:
         """
         Example from indices so that it is no more than max length.
         Use previous indices from cache.
@@ -635,7 +679,7 @@ class ConfusionMatrix(object):
             labels = self.labels
         return labels
 
-    def _get_matrix(self):
+    def _get_matrix(self, labels: Optional[List[str]] = None) -> List[Any]:
         matrix = None
         if self._matrix is not None:
             if self.selected is not None:
@@ -644,13 +688,15 @@ class ConfusionMatrix(object):
                     for row in self.selected
                 ]
             else:
-                matrix = self._expand_dict(self._matrix, default=0)
+                matrix = self._expand_dict(self._matrix, default=0, labels=labels)
         return matrix
 
-    def _get_example_matrix(self):
+    def _get_example_matrix(self, labels: Optional[List[str]] = None) -> List[Any]:
         smatrix = None
         if self._example_matrix is not None and len(self._example_matrix) > 0:
-            smatrix = self._expand_dict(self._example_matrix, default=None)
+            smatrix = self._expand_dict(
+                self._example_matrix, default=None, labels=labels
+            )
             if self.selected is not None:
                 smatrix = [
                     [smatrix[row][col] for col in self.selected]
@@ -658,19 +704,31 @@ class ConfusionMatrix(object):
                 ]
         return smatrix
 
-    def _expand_dict(self, matrix_dict, default=None):
-        # type: (Dict, Union[int, None]) -> List
+    def _expand_dict(
+        self,
+        matrix_dict: Dict,
+        default: Union[int, None] = None,
+        labels: Optional[List[str]] = None,
+    ) -> List[Any]:
         """
         Expand the dictionary representation into a full matrix.
         """
         n = self._dimension
+        if labels is not None and len(labels) > self._dimension:
+            n = len(labels)
         matrix = [[default for y in range(n)] for x in range(n)]
-        for (x, y) in matrix_dict:
+        for x, y in matrix_dict:
             matrix[x][y] = matrix_dict[(x, y)]
         return matrix
 
     def compute_matrix(
-        self, y_true, y_predicted, index_to_example_function=None, images=None, **kwargs
+        self,
+        y_true,
+        y_predicted,
+        index_to_example_function=None,
+        images=None,
+        images_annotations=None,
+        **kwargs
     ):
         """
         Compute the confusion matrix.
@@ -681,7 +739,18 @@ class ConfusionMatrix(object):
             y_predicted: list of vectors representing predicted
                 values, or a list of integers representing the output
             images: (optional) a list of data that can be passed to
-                Experiment.log_image()
+                [comet_ml.CometExperiment.log_image][]
+            images_annotations: (optional) Specifies annotation layers for images.
+                This parameter expects a list of lists, where each inner list defines an
+                annotation layer, potentially containing bounding boxes and/or polygons.
+
+                For a comprehensive understanding of how annotations are utilized and structured,
+                please consult the documentation for `Experiment.log_image()`.
+
+                **Important Note for Custom `index_to_example_function` Implementations:**
+                If you provide your own implementation for `index_to_example_function` (bypassing the default),
+                it is critical to ensure that every `images_annotations` entry is consistently passed alongside its corresponding image within each `log_image()` call.
+                Furthermore, the metadata associated with these annotations must precisely align with the expectations and requirements of the `Experiment.log_image()` call.
             index_to_example_function: (optional) a function
                 that takes an index and returns either
                 a number, a string, a URL, or a {"sample": str,
@@ -694,6 +763,7 @@ class ConfusionMatrix(object):
             y_true and y_predicted, if they are vectors.
         """
         self.images = images
+        self.images_annotations = images_annotations
         if self.images is not None:
             if index_to_example_function is None:
                 index_to_example_function = self.image_index_to_example_function
@@ -733,17 +803,21 @@ class ConfusionMatrix(object):
             self.initialize()
 
         examples = defaultdict(set)  # type: Dict[Tuple[int, int], Set[int]]
-        for (i, (raw_x, raw_y)) in enumerate(zip(xs, ys)):
+        for i, (raw_x, raw_y) in enumerate(zip(xs, ys)):
             try:
                 x = convert_to_scalar(raw_x, dtype=int)  # type: int
             except TypeError:
-                LOGGER.warning("Invalid y_true value %r, ignoring it", raw_x)
+                LOGGER.warning(
+                    CONFUSION_MATRIX_COMPUTE_MATRIX_INVALID_Y_TRUE_WARNING, raw_x
+                )
                 continue
 
             try:
                 y = convert_to_scalar(raw_y, dtype=int)  # type: int
             except TypeError:
-                LOGGER.warning("Invalid y_predictor value %r, ignoring it", raw_y)
+                LOGGER.warning(
+                    CONFUSION_MATRIX_COMPUTE_MATRIX_INVALID_Y_PREDICTOR_WARNING, raw_y
+                )
                 continue
 
             # Add count to cell:
@@ -751,6 +825,9 @@ class ConfusionMatrix(object):
             # Add index to cell:
             examples[(x, y)].add(i)
 
+        self._process_examples(examples, **kwargs)
+
+    def _process_examples(self, examples, **kwargs):
         # Example all cells that have items (reuse from cache/other cells if possible):
         for key in examples:
             x, y = key
@@ -775,9 +852,9 @@ class ConfusionMatrix(object):
             ordered_rows = sorted(correct_counts, key=lambda pair: pair[1])
             self.selected = [row[0] for row in ordered_rows[: self.max_categories]]
 
-        matrix = self._get_matrix()
-        smatrix = self._get_example_matrix()
         labels = self._get_labels()
+        matrix = self._get_matrix(labels)
+        smatrix = self._get_example_matrix(labels)
 
         if smatrix is None:
             self.type = None
@@ -785,7 +862,7 @@ class ConfusionMatrix(object):
         if matrix is not None:
             if len(matrix) != len(labels):
                 raise ValueError(
-                    "The length of labels does not match number of categories"
+                    CONFUSION_MATRIX_NUM_LABELS_NOT_EQUAL_CATEGORIES_EXCEPTION
                 )
 
         return {
