@@ -6,10 +6,9 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed without the express
-#  permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 #
 """
@@ -21,17 +20,21 @@ wrapping
 """
 import logging
 import multiprocessing
-import multiprocessing.util  # Force registering the atexit callback early
+import multiprocessing.util  # Force registering the atexit callback
 import os
 import os.path
+import queue
 import select
 import signal
 import sys
 import threading
 
-from six.moves import queue
-
-from .logging_messages import NATIVE_STD_WRAPPER_NOT_AVAILABLE, UNKOWN_STD_WRAPPER_SPEC
+from .logging_messages import (
+    CONSOLE_CONSUMER_THREAD_DID_NOT_FINISH_WARNING,
+    NATIVE_STD_WRAPPER_NOT_AVAILABLE,
+    UNKNOWN_STD_WRAPPER_SPEC,
+)
+from .messages import StandardOutputMessage
 
 # -------------------------------------------------
 # These are triggers for keras and tensorflow.keras
@@ -66,12 +69,20 @@ def stream_handler_set_stream(stream_handler_instance, stream):
     return result
 
 
+def _isatty(stream):
+    try:
+        return stream.isatty()
+    except Exception:
+        return False
+
+
 class FakeStd(object):
     """A fake Std file-like that sends every line to a handler."""
 
-    def __init__(self, handler, original):
+    def __init__(self, handler, original, isatty):
         self.__wrapped__ = original
         self.handler = handler
+        self._isatty = isatty
 
     def write(self, line):
         """
@@ -104,7 +115,7 @@ class FakeStd(object):
             pass
 
     def isatty(self):
-        return False
+        return self._isatty
 
     def close(self):
         self.flush()
@@ -148,12 +159,12 @@ class BaseStdWrapper(object):
 
     def __enter__(self):
         if self.stdout and self.stdout_handler:
-            self._stdout = FakeStd(self.stdout_handler, sys.stdout)
+            self._stdout = FakeStd(self.stdout_handler, sys.stdout, _isatty(sys.stdout))
             self._old_stdout = sys.stdout
             sys.stdout = self._stdout
 
         if self.stderr and self.stderr_handler:
-            self._stderr = FakeStd(self.stderr_handler, sys.stderr)
+            self._stderr = FakeStd(self.stderr_handler, sys.stderr, _isatty(sys.stderr))
             self._old_stderr = sys.stderr
             sys.stderr = self._stderr
 
@@ -183,6 +194,8 @@ class BaseStdWrapper(object):
                     self.hooked_handlers.append((logger, handler, current_stream))
 
     def __exit__(self, exception_type, exception_value, traceback):
+        LOGGER.debug("Cleaning BaseStdWrapper")
+
         if self.stdout and self._old_stdout:
             sys.stdout = self._old_stdout
             self._old_stdout = None
@@ -206,9 +219,9 @@ class BaseStdWrapper(object):
 
 
 def start_tee_process(*args, **kwargs):
-    process = multiprocessing.Process(target=_tee_process, args=args, kwargs=kwargs)
-    # Python 2 don't accept daemon as an __init__ argument
-    process.daemon = True
+    process = multiprocessing.Process(
+        target=_tee_process, daemon=True, args=args, kwargs=kwargs
+    )
     process.start()
     return process
 
@@ -262,21 +275,36 @@ class Unbuffered(object):
     writing.
     """
 
-    def __init__(self, stream):
+    def __init__(self, stream, isatty):
         self.__wrapped__ = stream
+        self._isatty = isatty
 
     def write(self, data):
-        self.__wrapped__.write(data)
-        self.__wrapped__.flush()
+        try:
+            self.__wrapped__.write(data)
+            self.__wrapped__.flush()
+        except Exception:
+            # ignore
+            pass
 
     def writelines(self, datas):
-        self.__wrapped__.writelines(datas)
-        self.__wrapped__.flush()
+        try:
+            self.__wrapped__.writelines(datas)
+            self.__wrapped__.flush()
+        except Exception:
+            # ignore
+            pass
+
+    def isatty(self):
+        return self._isatty
 
     def __getattr__(self, attr):
         # We cannot use wrapt in Python2 around sys.stdout/sys.stderr because
         # they are files and we cannot set custom attributes to files in Python
         # 2
+        if attr == "isatty":
+            return self.isatty
+
         return getattr(self.__wrapped__, attr)
 
 
@@ -367,8 +395,13 @@ try:
 
             _setup_fd(pipe_out)
 
+            # pipe_out, control_pipe, real_out_fd, name, queue
             process = start_tee_process(
-                pipe_out, control_pipe, save_fd, name, self.queue
+                pipe_out=pipe_out,
+                control_pipe=control_pipe,
+                real_out_fd=save_fd,
+                name=name,
+                queue=self.queue,
             )
 
             return process
@@ -378,24 +411,27 @@ try:
             self.queue = multiprocessing.Queue()
 
             if self.stdout:
+                isatty = _isatty(sys.stdout)
                 self.stdout_process = self._prepare_fd_and_process(
-                    "stdout", self.stdout_control_out
+                    "stdout", control_pipe=self.stdout_control_out
                 )
                 # Patch Python stdout to force buffer flush
                 self.original_stdout = sys.stdout
-                sys.stdout = Unbuffered(sys.stdout)
+                sys.stdout = Unbuffered(sys.stdout, isatty)
 
             if self.stderr:
+                isatty = _isatty(sys.stderr)
                 self.stderr_process = self._prepare_fd_and_process(
-                    "stderr", self.stderr_control_out
+                    "stderr", control_pipe=self.stderr_control_out
                 )
                 # Patch Python stderr to force buffer flush
                 self.original_stderr = sys.stderr
-                sys.stderr = Unbuffered(sys.stderr)
+                sys.stderr = Unbuffered(sys.stderr, isatty)
 
             # Setup and start the thread reading the queue
-            self.consumer_thread = threading.Thread(target=self.consume_queue)
-            self.consumer_thread.daemon = True
+            self.consumer_thread = threading.Thread(
+                target=self.consume_queue, daemon=True
+            )
             self.consumer_thread.start()
 
         @staticmethod
@@ -470,7 +506,7 @@ try:
             if self.consumer_thread:
                 self.consumer_thread.join(5)
                 if self.consumer_thread.is_alive():
-                    LOGGER.warning("Consumer thread did not finish")
+                    LOGGER.warning(CONSOLE_CONSUMER_THREAD_DID_NOT_FINISH_WARNING)
 
             os.close(self.stdout_control_out)
             os.close(self.stdout_control_in)
@@ -527,19 +563,27 @@ class StdLogger(object):
             LOGGER.debug("Error saving stderr message", exc_info=True)
 
     def handler(self, data, std_name):
+        # do not stream DEBUG messages to avoid infinite recursion
+        if "DEBUG" in data:
+            return
+
         if not self.experiment:
             return
 
-        payload = self.experiment._create_message()
-
         if std_name == "stdout":
-            payload.set_stdout(data)
+            stderr = False
         elif std_name == "stderr":
-            payload.set_stderr(data)
+            stderr = True
         else:
             raise NotImplementedError()
 
-        self.streamer.put_message_in_q(payload)
+        message = StandardOutputMessage(
+            output=data,
+            stderr=stderr,
+            context=self.experiment.context,
+        )
+
+        self.streamer.put_message_in_q(message)
 
 
 def get_std_logger(wrapper_spec, streamer):
@@ -558,6 +602,6 @@ def get_std_logger(wrapper_spec, streamer):
             LOGGER.warning(NATIVE_STD_WRAPPER_NOT_AVAILABLE)
 
     else:
-        LOGGER.warning(UNKOWN_STD_WRAPPER_SPEC, wrapper_spec)
+        LOGGER.warning(UNKNOWN_STD_WRAPPER_SPEC, wrapper_spec)
 
     return StdLogger(streamer, BaseStdWrapper)
