@@ -6,21 +6,20 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed without the express
-#  permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 
-from __future__ import print_function
-
 import json
 import logging
 import math
 import os
 
 from ._typing import List, Optional
+from .assets.assets_helper import asset_download_rest_path
 from .convert_utils import HAS_NUMPY, fast_flatten, numpy
+from .file_utils import expand_user_home_path
 from .utils import log_once_at_level
 
 LOGGER = logging.getLogger(__name__)
@@ -31,39 +30,78 @@ class Embedding(object):
     Data structure for holding embedding template info.
     """
 
+    DEFAULT_TITLE = "Comet Embedding"
+
     def __init__(
         self,
-        vector_url,
-        vector_shape,
-        metadata_url,
-        sprite_url=None,
-        image_size=None,
-        title="Comet Embedding",
+        vector_url: str,
+        vector_shape: List[int],
+        metadata_url: str,
+        sprite_url: Optional[str] = None,
+        image_size: Optional[List[int]] = None,
+        title: str = DEFAULT_TITLE,
+        vector_asset_id: Optional[str] = None,
+        metadata_asset_id: Optional[str] = None,
+        sprite_asset_id: Optional[str] = None,
+        experiment_key: Optional[str] = None,
     ):
         """
-        The URL's and sizes for all of the Embedding'
-        resources.
+        The URL's and sizes for all Embedding's resources.
         """
+        if vector_url is None and vector_asset_id is None:
+            raise ValueError("Either vector_url or vector_asset_id must be provided")
+
+        if metadata_url is None and metadata_asset_id is None:
+            raise ValueError(
+                "Either metadata_url or metadata_asset_id must be provided"
+            )
+
         self.vector_url = vector_url
         self.vector_shape = vector_shape
         self.metadata_url = metadata_url
         self.sprite_url = sprite_url
         self.image_size = image_size
         self.title = title
+        self.vector_asset_id = vector_asset_id
+        self.metadata_asset_id = metadata_asset_id
+        self.sprite_asset_id = sprite_asset_id
+        self.experiment_key = experiment_key
 
     def to_json(self):
         """
         Return a JSON representation of the embedding
         """
+        if self.vector_url is not None:
+            tensor_path = self.vector_url
+        else:
+            tensor_path = asset_download_rest_path(
+                asset_id=self.vector_asset_id, experiment_key=self.experiment_key
+            )
+
+        if self.metadata_url is not None:
+            metadata_path = self.metadata_url
+        else:
+            metadata_path = asset_download_rest_path(
+                asset_id=self.metadata_asset_id, experiment_key=self.experiment_key
+            )
+
+        image_path = None
+        if self.sprite_url is not None:
+            image_path = self.sprite_url
+        elif self.sprite_asset_id is not None:
+            image_path = asset_download_rest_path(
+                asset_id=self.sprite_asset_id, experiment_key=self.experiment_key
+            )
+
         template = {
             "tensorName": self.title,
             "tensorShape": list(self.vector_shape),
-            "tensorPath": self.vector_url,
-            "metadataPath": self.metadata_url,
+            "tensorPath": tensor_path,
+            "metadataPath": metadata_path,
         }
-        if self.sprite_url is not None:
+        if image_path is not None:
             template["sprite"] = {
-                "imagePath": self.sprite_url,
+                "imagePath": image_path,
                 "singleImageDim": list(self.image_size),
             }
         return template
@@ -74,7 +112,7 @@ class Histogram(object):
     Data structure for holding a counts of values. Creates an
     exponentially-distributed set of bins.
 
-    See also [`Experiment.log_histogram`](#experimentlog_histogram)
+    See also [comet_ml.CometExperiment.log_histogram][]
     """
 
     def __init__(self, start=1e-12, stop=1e20, step=1.1, offset=0):
@@ -208,6 +246,9 @@ class Histogram(object):
         midpoint = len(self.counts) // 2
         base = numpy.log(self.step)
 
+        # Round values to zero that are too small:
+        values[numpy.isclose(values, 0, atol=1e-20)] = 0.0
+
         # Positive values (values > offset)
 
         # The filter is an array of boolean same size as values
@@ -360,7 +401,7 @@ class Histogram(object):
         ```
         """
         if isinstance(data, str):
-            filename = os.path.expanduser(data)
+            filename = expand_user_home_path(data)
             if os.path.isfile(filename):
                 with open(filename) as fp:
                     histogram_json = json.load(fp)
@@ -383,7 +424,7 @@ class Histogram(object):
             histogram = Histogram(
                 data["start"], data["stop"], data["step"], data["offset"]
             )
-            for (i, count) in data["index_values"]:
+            for i, count in data["index_values"]:
                 histogram.counts[i] = count
             if verbose:
                 histogram.display()
@@ -523,7 +564,7 @@ class Histogram(object):
         Example:
 
         ```
-        >>> from comet_ml.utils import Histogram
+        >>> from comet_ml.data_structure import Histogram
         >>> import random
         >>> history = Histogram()
         >>> values = [random.random() for x in range(10000)]
