@@ -6,10 +6,9 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed without the express
-#  permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 
 """
@@ -19,22 +18,28 @@ This module contains the functions dedicated to logging the environment
 information
 
 """
-
+import functools
 import inspect
+import json
 import logging
 import os
 import platform
 import socket
 import sys
 
-import pkg_resources
-import requests
-import six
-from six import StringIO
-from six.moves.urllib.parse import urlparse
+if sys.version_info[:2] > (3, 9):
+    import importlib.metadata as importlib_metadata
+else:
+    # importlib_metadata for versions <= (3, 9) doesn't have names for distributions
+    import pkg_resources
 
-from ._typing import Any, Dict, List, Optional, Tuple
-from .connection import get_backend_address
+from io import StringIO
+from urllib.parse import urlparse
+
+import requests
+
+from ._typing import Any, Dict, List, Optional, Tuple, Union
+from .config import get_backend_address
 from .logging_messages import (
     LOG_CONDA_ENV_ERROR,
     LOG_CONDA_ENV_RETURN_CODE,
@@ -48,32 +53,36 @@ from .utils import get_comet_version, get_user, subprocess_run_and_check
 
 LOGGER = logging.getLogger(__name__)
 
+PYTEST_ENV = "pytest" in sys.modules  # TODO: check if there are better alternatives
 
-def get_pid():
+CONDA_ENV_TIMEOUT = 60
+
+
+def get_pid() -> int:
     return os.getpid()
 
 
-def get_hostname():
+def get_hostname() -> str:
     return socket.gethostname()
 
 
-def get_os():
+def get_os() -> str:
     return platform.platform(aliased=True)
 
 
-def get_os_type():
+def get_os_type() -> str:
     return platform.system()
 
 
-def get_python_version_verbose():
+def get_python_version_verbose() -> str:
     return sys.version
 
 
-def get_python_version():
+def get_python_version() -> str:
     return platform.python_version()
 
 
-def get_ip():
+def get_ip() -> Optional[str]:
     try:
         s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
         server_address = get_backend_address()
@@ -92,55 +101,17 @@ def get_ip():
         return None
 
 
-def get_command():
+def get_command() -> List[str]:
     return sys.argv
 
 
-def get_env_variables(blacklist, os_environ):
-    # type: (List[str], Dict[str, str]) -> Dict[str, str]
-
-    lowercase_blacklist = [pattern.lower() for pattern in blacklist]
-
-    def filter_env_variable(env_variable_name):
-        # type: (str) -> bool
-        """Filter environment variable names containing at least one of the blacklist pattern"""
-        lowercase_env_variable_name = env_variable_name.lower()
-        for pattern in lowercase_blacklist:
-            if pattern in lowercase_env_variable_name:
-                return False
-
-        return True
-
-    return {key: value for key, value in os_environ.items() if filter_env_variable(key)}
-
-
-def get_env_details():
-    return {
-        "pid": get_pid(),
-        "hostname": get_hostname(),
-        "os": get_os(),
-        "os_type": get_os_type(),
-        "python_version_verbose": get_python_version_verbose(),
-        "python_version": get_python_version(),
-        "user": get_user(),
-        "ip": get_ip(),
-        "command": get_command(),
-        "python_exe": sys.executable,
-    }
-
-
-def get_env_details_message(env_blacklist, include_env):
-    # type: (List[str], bool) -> SystemDetailsMessage
-    if include_env:
-        env = get_env_variables(env_blacklist, os.environ)
-    else:
-        env = None
-
+def get_env_details_message() -> SystemDetailsMessage:
     os_type, _, os_release, _, machine, processor = platform.uname()
 
-    return SystemDetailsMessage(
+    return SystemDetailsMessage.create(
+        context=None,
         command=get_command(),
-        env=env,
+        env=None,
         hostname=get_hostname(),
         ip=get_ip(),
         machine=machine,
@@ -156,9 +127,8 @@ def get_env_details_message(env_blacklist, include_env):
     )
 
 
-def _get_non_pii_env_details(comet_url):
-    # type: (str) -> Dict[str, Any]
-    """Return environment details without PII, usefull to send to external services like error tracking"""
+def _get_non_pii_env_details(comet_url: str) -> Dict[str, Any]:
+    """Return environment details without PII, useful to send to external services like error tracking"""
 
     return {
         "pid": get_pid(),
@@ -173,13 +143,19 @@ def _get_non_pii_env_details(comet_url):
     }
 
 
-def get_caller_file_path():
-    # type: () -> Optional[Tuple[str, str]]
+def get_caller_file_path() -> Optional[Tuple[str, str]]:
     """Returns the module name and file path from the first caller that isn't in the blacklisted
     list. If none is found, returns None
     """
     # Don't get source code from these libraries:
-    ignore_list = ("comet_ml", "mlflow", "transformers", "ludwig", "pytorch_lightning")
+    ignore_list = (
+        "comet_ml",
+        "mlflow",
+        "transformers",
+        "ludwig",
+        "pytorch_lightning",
+        "metaflow",
+    )
 
     for frame in inspect.stack(context=1):
         module = inspect.getmodule(frame[0])
@@ -197,8 +173,7 @@ def get_caller_file_path():
     return None
 
 
-def get_ipython_source_code():
-    # type: () -> str
+def get_ipython_source_code() -> str:
     """
     Get the IPython source code from the history. Assumes that this
     method is run in a jupyter environment.
@@ -239,12 +214,11 @@ def get_ipython_cells():
     return cells
 
 
-def _format_text(text):
-    # type: (Any) -> List[str]
+def _format_text(text: Any) -> List[str]:
     """
     Format multi-line text in the ipython style.
     """
-    if not isinstance(text, six.string_types):
+    if not isinstance(text, str):
         text = repr(text)
 
     lines = text.splitlines(True)  # type: List[str]
@@ -253,8 +227,7 @@ def _format_text(text):
     return lines
 
 
-def _format_output(output, execution_count):
-    # type: (Any, int) -> Optional[Dict]
+def _format_output(output: Any, execution_count: int) -> Optional[Dict]:
     """
     Get the mime bundle, etc. for each output form.
     """
@@ -277,8 +250,7 @@ def _format_output(output, execution_count):
         }
 
 
-def create_notebook(cells):
-    # type: (List[Tuple[Optional[str], Optional[Any]]]) -> Dict
+def create_notebook(cells: List[Tuple[Optional[str], Optional[Any]]]) -> Dict[str, Any]:
     """
     From a list of (input, output) tuples, create a JSON notebook.
     """
@@ -302,13 +274,11 @@ def create_notebook(cells):
     return {"nbformat": 4, "nbformat_minor": 0, "metadata": {}, "cells": final_cells}
 
 
-def get_ipython_notebook():
-    # type: () -> Dict
+def get_ipython_notebook() -> Dict[str, Any]:
     return create_notebook(get_ipython_cells())
 
 
-def postprocess_gcp_cloud_metadata(cloud_metadata):
-    # type: (Dict[str, Any]) -> Dict[str, Any]
+def postprocess_gcp_cloud_metadata(cloud_metadata: Dict[str, Any]) -> Dict[str, Any]:
 
     # Attributes contains custom metadata and also contains Kubernetes config,
     # startup script and secrets, filter it out
@@ -335,8 +305,8 @@ CLOUD_METADATA_MAPPING = {
 }
 
 
-def get_env_cloud_details(timeout=1):
-    # type: (int) -> Optional[Any]
+@functools.lru_cache(0 if PYTEST_ENV else 128)
+def get_env_cloud_details(timeout: int = 1) -> Optional[Any]:
     for provider in CLOUD_METADATA_MAPPING.keys():
         try:
             params = CLOUD_METADATA_MAPPING[provider]
@@ -359,43 +329,81 @@ def get_env_cloud_details(timeout=1):
     return None
 
 
-def _get_pip_packages():
-    # type: () -> List[str]
-    return sorted(["%s==%s" % (i.key, i.version) for i in pkg_resources.working_set])
+@functools.lru_cache(0 if PYTEST_ENV else 128)
+def get_pip_packages(
+    as_tuples: bool = False,
+) -> Union[List[str], List[Tuple[str, str]]]:
+    if sys.version_info[:2] <= (3, 9):
+        result_tuples = sorted(
+            [(package.key, package.version) for package in pkg_resources.working_set]
+        )
+    else:
+        result_tuples = sorted(
+            [
+                (package.name.lower(), package.version)
+                for package in importlib_metadata.distributions()
+            ]
+        )
+
+    if as_tuples:
+        return result_tuples
+
+    return ["%s==%s" % (name, version) for name, version in result_tuples]
 
 
-def _in_conda_environment():
-    # type: () -> bool
+def _in_conda_environment() -> bool:
     return os.path.exists(os.path.join(sys.prefix, "conda-meta", "history"))
 
 
-def _in_pydev_console():
-    # type: () -> bool
+def _in_pydev_console() -> bool:
     return "PYDEVD_LOAD_VALUES_ASYNC" in os.environ
 
 
-def _get_conda_env():
-    # type: () -> Optional[bytes]
-    args = ["conda", "env", "export"]
-    timeout = 30
-    return subprocess_run_and_check(
-        args, timeout, LOG_CONDA_ENV_ERROR, LOG_CONDA_ENV_RETURN_CODE
+def _get_conda_env() -> Optional[str]:
+    args = ["conda", "env", "export", "--json"]
+
+    result = subprocess_run_and_check(
+        args, CONDA_ENV_TIMEOUT, LOG_CONDA_ENV_ERROR, LOG_CONDA_ENV_RETURN_CODE
     )
+    if result is None:
+        return None
+
+    try:
+        decoded = json.loads(result)
+        # Remove the variables key because it can contain secrets
+        decoded.pop("variables", None)
+        return json.dumps(decoded, indent=4, sort_keys=True)
+    except Exception:
+        LOGGER.warning(LOG_CONDA_ENV_ERROR, exc_info=True)
+        return None
 
 
-def _get_conda_explicit_packages():
-    # type: () -> Optional[bytes]
+def _get_conda_explicit_packages() -> Optional[bytes]:
     args = ["conda", "list", "--explicit", "--md5"]
-    timeout = 30
     return subprocess_run_and_check(
-        args, timeout, LOG_CONDA_PACKAGES_ERRORS, LOG_CONDA_PACKAGES_RETURN_CODE
+        args,
+        CONDA_ENV_TIMEOUT,
+        LOG_CONDA_PACKAGES_ERRORS,
+        LOG_CONDA_PACKAGES_RETURN_CODE,
     )
 
 
-def _get_conda_info():
-    # type: () -> Optional[bytes]
+def _get_conda_info() -> Optional[bytes]:
     args = ["conda", "info"]
-    timeout = 30
     return subprocess_run_and_check(
-        args, timeout, LOG_CONDA_INFO_ERRORS, LOG_CONDA_INFO_RETURN_CODE
+        args, CONDA_ENV_TIMEOUT, LOG_CONDA_INFO_ERRORS, LOG_CONDA_INFO_RETURN_CODE
     )
+
+
+def parse_log_other_instructions() -> Dict[str, str]:
+    result = {}
+    PREFIX = "comet_log_other_"
+    PREFIX_LENGTH = len(PREFIX)
+
+    for key, value in os.environ.items():
+        lowered_key = key.lower()
+        if lowered_key.startswith(PREFIX) and len(lowered_key) > PREFIX_LENGTH:
+            name = lowered_key[PREFIX_LENGTH:]
+            result[name] = value
+
+    return result
