@@ -6,10 +6,9 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2022 Comet ML INC
-#  This file can not be copied and/or distributed without the express
-#  permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 
 from types import TracebackType
@@ -17,6 +16,7 @@ from types import TracebackType
 from comet_ml import get_global_experiment
 
 from ._typing import Any, Callable, Optional, Type
+from .logging_messages import UNEXPECTED_CUSTOMER_ERROR
 
 ExceptionHookType = Callable[
     [Type[BaseException], BaseException, Optional[TracebackType]],
@@ -31,7 +31,9 @@ def _create_exception_hook(old_exception_hook):
         experiment = get_global_experiment()
 
         if experiment is not None:
-            experiment._report_experiment_error()
+            experiment._report_experiment_error(
+                UNEXPECTED_CUSTOMER_ERROR % exception_value, has_crashed=True
+            )
 
         return old_exception_hook(exception_type, exception_value, traceback)
 
