@@ -6,26 +6,24 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed without the express
-#  permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 
 """ This module handles asynchronous downloading of the files."""
 
 import logging
 import math
-import multiprocessing.pool
+from typing import Any, Callable, Dict, List, NamedTuple, Optional, Tuple
 
-from ._typing import Any, Callable, List, Optional, Tuple
 from .logging_messages import (
     FILE_DOWNLOAD_MANAGER_COMPLETED,
     FILE_DOWNLOAD_MANAGER_MONITOR_FIRST_MESSAGE,
     FILE_DOWNLOAD_MANAGER_MONITOR_PROGRESSION,
     FILE_DOWNLOAD_MANAGER_MONITOR_PROGRESSION_UNKNOWN_ETA,
 )
-from .parallel_utils import get_thread_pool
+from .thread_pool import Future, get_thread_pool
 from .utils import format_bytes, get_time_monotonic
 
 LOGGER = logging.getLogger(__name__)
@@ -36,12 +34,11 @@ class FileDownloadSizeMonitor(object):
 
     __slots__ = ["total_size", "bytes_written"]
 
-    def __init__(self):
+    def __init__(self) -> None:
         self.total_size = None
         self.bytes_written = 0
 
-    def monitor_callback(self, bytes_written):
-        # type: (int) -> None
+    def monitor_callback(self, bytes_written: int) -> None:
         if bytes_written is not None:  # Python2.7 compatibility check
             self.bytes_written += bytes_written
 
@@ -49,30 +46,28 @@ class FileDownloadSizeMonitor(object):
 class DownloadResult(object):
     """The multiprocessing.pool.AsyncResult wrapper providing additional information about download task"""
 
-    def __init__(self, async_result, monitor):
-        # type: (multiprocessing.pool.AsyncResult, FileDownloadSizeMonitor) -> None
-        self.async_result = async_result
+    def __init__(self, future: Future, monitor: FileDownloadSizeMonitor) -> None:
+        self.future = future
         self.monitor = monitor
 
-    def ready(self):
-        # type: () -> bool
-        """Allows to check if wrapped AsyncResult is ready"""
-        return self.async_result.ready()
+    def ready(self) -> bool:
+        """Allows to check if wrapped Future successfully finished"""
+        return self.future.done()
 
-    def get(self, timeout=None):
-        # type: (Optional[int]) -> Any
-        """Return the result when it arrives from the wrapped multiprocessing.pool.AsyncResult.
+    def get(self, timeout: Optional[int] = None) -> Any:
+        """Return the result when it arrives from the wrapped Future.
         If timeout is not None and the result does not arrive within timeout seconds then
-        multiprocessing.TimeoutError is raised. If the remote call raised an exception then
+        TimeoutError is raised. If the remote call raised an exception then
         that exception will be reraised by get()."""
-        return self.async_result.get(timeout)
+        return self.future.result(timeout)
 
 
 class FileDownloadManager(object):
     """The manager to handle downloading of the files and reporting the progress"""
 
-    def __init__(self, worker_cpu_ratio, worker_count=None):
-        # type: (int, Optional[int]) -> None
+    def __init__(
+        self, worker_cpu_ratio: int, worker_count: Optional[int] = None
+    ) -> None:
         """Creates new instance with specified ration of workers per CPU. This will instantiate the thread pool for
         asynchronous downloads processing in multiple processes.
         Args:
@@ -80,7 +75,7 @@ class FileDownloadManager(object):
         """
         self.download_results = []  # type: List[DownloadResult]
 
-        pool_size, cpu_count, self._pool = get_thread_pool(
+        pool_size, cpu_count, self._executor = get_thread_pool(
             worker_cpu_ratio, worker_count
         )
 
@@ -92,8 +87,12 @@ class FileDownloadManager(object):
             worker_count,
         )
 
-    def download_file_async(self, download_func, estimated_size=None, **kwargs):
-        # type: (Callable, int, Optional[Any]) -> DownloadResult
+    def download_file_async(
+        self,
+        download_func: Callable,
+        estimated_size: Optional[int] = None,
+        **kwargs: Optional[Any]
+    ) -> DownloadResult:
         """Registers file to be downloaded asynchronously using specified download processing function.
         Args:
             download_func: the function to maintain file downloading routines.
@@ -104,18 +103,16 @@ class FileDownloadManager(object):
             monitor.total_size = estimated_size
 
         kwargs["_monitor"] = monitor
-        async_result = self._pool.apply_async(download_func, args=[], kwds=kwargs)
-        result = DownloadResult(async_result=async_result, monitor=monitor)
+        future = self._executor.submit(download_func, **kwargs)
+        result = DownloadResult(future=future, monitor=monitor)
         self.download_results.append(result)
         return result
 
-    def all_done(self):
-        # type: () -> bool
+    def all_done(self) -> bool:
         """Allows to check if all downloads completed"""
         return all(result.ready() for result in self.download_results)
 
-    def remaining_data(self):
-        # type: () -> (Tuple[int, int, int])
+    def remaining_data(self) -> Tuple[int, int, int]:
         """Calculates the number of remaining files, bytes, and total size to
         be downloaded."""
         remaining_downloads = 0
@@ -136,31 +133,27 @@ class FileDownloadManager(object):
 
         return remaining_downloads, remaining_bytes_to_download, total_size
 
-    def remaining_downloads(self):
-        # type: () -> int
+    def remaining_downloads(self) -> int:
         """Returns number of files to be downloaded"""
-        return len([1 for result in self.download_results if result.ready() is False])
+        status_list = [result.ready() for result in self.download_results]
+        return status_list.count(False)
 
-    def close(self):
-        # type: () -> None
-        self._pool.close()
+    def close(self) -> None:
+        self._executor.close()
 
-    def join(self):
-        # type: () -> None
-        self._pool.join()
+    def join(self) -> None:
+        self._executor.join()
 
 
 class FileDownloadManagerMonitor(object):
     """The monitor to log download progress of the associated FileDownloadManager"""
 
-    def __init__(self, file_download_manager):
-        # type: (FileDownloadManager) -> None
+    def __init__(self, file_download_manager: FileDownloadManager) -> None:
         self.file_download_manager = file_download_manager
         self.last_remaining_bytes = 0
         self.last_remaining_downloads_log_time = None  # type: Optional[float]
 
-    def log_remaining_downloads(self):
-        # type: () -> None
+    def log_remaining_downloads(self) -> None:
         (
             downloads,
             remaining_bytes,
@@ -207,7 +200,17 @@ class FileDownloadManagerMonitor(object):
         self.last_remaining_bytes = remaining_bytes
         self.last_remaining_downloads_log_time = current_time
 
-    def all_done(self):
-        # type: () -> bool
+    def all_done(self) -> bool:
         """Checks if all downloads completed by monitored download manager."""
         return self.file_download_manager.all_done()
+
+
+class DownloadResultHolder(NamedTuple):
+    download_result: DownloadResult
+    asset_filename: str
+    asset_path: str
+    asset_metadata: Dict[str, Any]
+    asset_id: str
+    asset_synced: bool
+    asset_type: str
+    asset_remote_uri: str
