@@ -6,25 +6,28 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed without
-#  the express permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 
 import copy
 import logging
 
 from .._typing import Any, Dict, Optional, Tuple
-from ..experiment import BaseExperiment
+from ..experiment import CometExperiment
 from ..logging_messages import GET_CALLBACK_FAILURE
 from ..monkey_patching import check_module
 
 LOGGER = logging.getLogger(__name__)
 
 
-def train_logger(experiment, original, *args, **kwargs):
-    # type: (BaseExperiment, Any, Any, Any) -> Optional[Tuple[Tuple[Any, ...], Dict[Any, Any]]]
+def train_logger(
+    experiment: CometExperiment,
+    original: Any,
+    *args: Any,
+    **kwargs: Any,
+) -> Optional[Tuple[Tuple[Any, ...], Dict[Any, Any]]]:
 
     try:
         callback = experiment.get_callback("lightgbm")
