@@ -6,10 +6,9 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed without
-#  the express permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 
 import io
@@ -17,7 +16,8 @@ import json
 import logging
 
 from .._typing import Any
-from ..experiment import BaseExperiment
+from ..experiment import CometExperiment
+from ..messages import ParameterMessage
 from ..monkey_patching import check_module
 from .logger_utils import get_argument_bindings
 
@@ -29,7 +29,7 @@ class ProphetFigureLogger(object):
         self.title = title
 
     def after(self, experiment, original, results, *args, **kwargs):
-        # type: (BaseExperiment, Any, Any, Any, Any) -> None
+        # type: (CometExperiment, Any, Any, Any, Any) -> None
         if experiment.config.get_bool(None, "comet.auto_log.figures"):
             experiment._storage["prophet"]["counter"] += 1
             experiment._log_figure(
@@ -42,14 +42,19 @@ class ProphetFigureLogger(object):
 
 
 def prophet_constructor(experiment, original, results, *args, **kwargs):
-    # type: (BaseExperiment, Any, Any, Any, Any) -> None
+    # type: (CometExperiment, Any, Any, Any, Any) -> None
     if experiment.auto_param_logging:
         parameters = get_argument_bindings(original, args, kwargs)
-        experiment._log_parameters(parameters, framework="prophet")
+        experiment._log_parameters(
+            parameters,
+            framework="prophet",
+            source=ParameterMessage.source_autologger,
+            flatten_nested=False,
+        )
 
 
 def prophet_fit(experiment, original, results, *args, **kwargs):
-    # type: (BaseExperiment, Any, Any, Any, Any) -> None
+    # type: (CometExperiment, Any, Any, Any, Any) -> None
     # Note: Experiment.log_graph controls logging the model here:
     if experiment.log_graph and not experiment._storage["prophet"]["internal"]:
         try:
@@ -75,12 +80,12 @@ def prophet_fit(experiment, original, results, *args, **kwargs):
 
 
 def prophet_cross_validation_before(experiment, original, *args, **kwargs):
-    # type: (BaseExperiment, Any, Any, Any) -> None
+    # type: (CometExperiment, Any, Any, Any) -> None
     experiment._storage["prophet"]["internal"] = True
 
 
 def prophet_cross_validation_after(experiment, original, results, *args, **kwargs):
-    # type: (BaseExperiment, Any, Any, Any, Any) -> None
+    # type: (CometExperiment, Any, Any, Any, Any) -> None
     experiment._storage["prophet"]["internal"] = False
 
 
