@@ -6,10 +6,9 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed without the express
-#  permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 
 """ The Pytorch auto-logger is logging the following elements:
@@ -65,39 +64,77 @@ return scaled_loss. But that also means that we are leaking references to the or
 case.
 """
 
+import inspect
 import logging
 
+import comet_ml.event_tracker
+import comet_ml.inspect
+
+import box
 import wrapt
 
 from .._typing import Any
-from ..experiment import BaseExperiment
+from ..experiment import CometExperiment
 from ..monkey_patching import check_module
 
 LOGGER = logging.getLogger(__name__)
 
 LOSS_BATCH_SIZE = 10
 
+KNOWN_TORCH_SAVE_CALLERS = (
+    "comet_ml",
+    "torch",
+    "pytorch_lightning",
+    "transformers",
+    "torchtext",
+    "ray",
+    "torch_geometric",
+    "fastai",
+    "lightning_fabric",
+    "neuralprophet",
+)
 
-def _get_loss(loss_backward, experiment):
-    # type: (Any, BaseExperiment) -> Any
+
+def _initialize_pytorch_localstorage(experiment_localstorage) -> None:
+    """
+    This is a protection from AttributeError which may happen
+    if torch autologger will try to log value from the new thread.
+    Before this fix the pytorch attribute initialization happened
+    in CometExperiment.__init__, so if someone created an experiment in
+    one thread but trained model in another - logging failed.
+    """
+    if not hasattr(experiment_localstorage, "pytorch"):
+        experiment_localstorage.pytorch = {"amp_loss_mapping": {}}
+
+
+def _get_loss(loss_backward: Any, experiment: CometExperiment) -> Any:
     """Returns the right loss tensor based on the loss tensor object where
     backward has been called. The right loss might be the unscaled one when
-    using APEX or Pytorch 1.6 AMP
+    using APEX, Pytorch 1.6 AMP or accelerate.Accelerator.
     """
-    if experiment.alive:
-        return experiment._localstorage.pytorch["amp_loss_mapping"].get(
-            id(loss_backward), loss_backward
-        )
-    else:
-        # Do not warn the user, it might send too many log messages
+    if not experiment.alive:
         LOGGER.debug("Attempting to get amp_loss_mapping for closed experiment")
         return None
 
+    _initialize_pytorch_localstorage(experiment._localstorage)
 
-def _add_amp_loss_mapping(scaled_loss, unscaled_loss, experiment):
-    # type: (int, Any, BaseExperiment) -> None
+    accelerator_step, accelerator_loss = experiment._localstorage.pytorch.get(
+        "accelerator_step_and_loss", (None, None)
+    )
+    if accelerator_step == experiment.curr_step:
+        loss_backward = accelerator_loss
+
+    return experiment._localstorage.pytorch["amp_loss_mapping"].get(
+        id(loss_backward), loss_backward
+    )
+
+
+def _add_amp_loss_mapping(
+    scaled_loss: int, unscaled_loss: Any, experiment: CometExperiment
+) -> None:
     # First argument is the scaled_loss id
     if experiment.alive:
+        _initialize_pytorch_localstorage(experiment._localstorage)
         experiment._localstorage.pytorch["amp_loss_mapping"][
             scaled_loss
         ] = unscaled_loss
@@ -106,8 +143,7 @@ def _add_amp_loss_mapping(scaled_loss, unscaled_loss, experiment):
         LOGGER.debug("Attempting to update amp_loss_mapping for closed experiment")
 
 
-def _clean_amp_loss_mapping(scaled_loss, experiment):
-    # type: (int, BaseExperiment) -> None
+def _clean_amp_loss_mapping(scaled_loss: int, experiment: CometExperiment) -> None:
     # First argument is the scaled_loss id
 
     # When an experiment is ended, the storage should be cleared. Be defensive and still try to
@@ -119,8 +155,7 @@ def _clean_amp_loss_mapping(scaled_loss, experiment):
     pytorch_storage.get("amp_loss_mapping", {}).pop(scaled_loss, None)
 
 
-def _log_loss_value(arg_loss, experiment):
-    # type: (Any, BaseExperiment) -> None
+def _log_loss_value(arg_loss: Any, experiment: CometExperiment) -> None:
     if not experiment.auto_metric_logging:
         # auto logging is not enabled - just return
         return None
@@ -148,13 +183,22 @@ def _log_loss_value(arg_loss, experiment):
             )
 
 
-def tensor_backward(experiment, original, result, *args, **kwargs):
+def tensor_backward(
+    experiment: CometExperiment, original: Any, result: Any, *args: Any, **kwargs: Any
+) -> Any:
     # args[0] is self, the Tensor (loss):
     try:
+        it_is_not_initial_step = experiment._storage["torch"].get(
+            "initial_step_passed", False
+        )
+
         if experiment.curr_step is None:
-            experiment.set_step(0)
-        else:
-            experiment.set_step(experiment.curr_step + 1)
+            experiment._set_step(0)
+        elif it_is_not_initial_step:
+            experiment._set_step(experiment.curr_step + 1)
+
+        if experiment.curr_step == 0:
+            experiment._storage["torch"]["initial_step_passed"] = True
 
         if experiment.log_graph:
             model = experiment._storage["torch"]["model"]
@@ -178,7 +222,28 @@ def tensor_backward(experiment, original, result, *args, **kwargs):
     return result
 
 
-def model_constructor(experiment, original, *args, **kwargs):
+def before_accelerator_backward_hook(
+    experiment: CometExperiment, original: Any, *args: Any, **kwargs: Any
+) -> None:
+    _initialize_pytorch_localstorage(experiment._localstorage)
+    next_step = 0 if experiment.curr_step is None else experiment.curr_step + 1
+
+    original_loss = args[1]
+
+    if experiment.alive:
+        experiment._localstorage.pytorch["accelerator_step_and_loss"] = (
+            next_step,
+            original_loss,
+        )
+    else:
+        LOGGER.debug(
+            "Attempting to update accelerator_step_and_loss for closed experiment"
+        )
+
+
+def model_constructor(
+    experiment: CometExperiment, original: Any, *args: Any, **kwargs: Any
+) -> None:
     ## Assume the first one is the model:
     try:
         model = experiment._storage["torch"]["model"]
@@ -228,6 +293,29 @@ def amp_scale_loss_hook(
     _add_amp_loss_mapping(id(return_value), original_loss, experiment)
 
 
+def _parse_callers(callstack_frames):
+    this_frame, monkeypatch_frame, original_caller = callstack_frames[:3]
+    return box.Box(
+        this_frame=this_frame,
+        monkeypatch_frame=monkeypatch_frame,
+        original_caller=original_caller,
+    )
+
+
+def save_hook(experiment, original, result, *args, **kwargs):
+    callstack_frames = inspect.stack(context=1)
+    callers = _parse_callers(callstack_frames)
+    caller = comet_ml.inspect.identify_caller(
+        callers.original_caller,
+        KNOWN_TORCH_SAVE_CALLERS,
+        return_if_unknown="unknown",
+    )
+
+    comet_ml.event_tracker.register(
+        "torch.save-called-by-{}".format(caller), experiment.id
+    )
+
+
 def patch(module_finder):
     ## For testing:
     check_module("torch")
@@ -247,6 +335,13 @@ def patch(module_finder):
     module_finder.register_after(
         "torch.cuda.amp.grad_scaler", "GradScaler.scale", amp_scale_loss_hook
     )
+    module_finder.register_before(
+        "accelerate", "Accelerator.backward", before_accelerator_backward_hook
+    )
+
+    # Disabled due to performance impact in multiprocessing dataloader where
+    # this function is called for tensors serialization
+    # module_finder.register_after("torch", "save", save_hook)
 
 
 check_module("torch")
