@@ -6,16 +6,16 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed without
-#  the express permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 
 import logging
+from typing import Tuple
 
 from .._typing import Any
-from ..experiment import BaseExperiment
+from ..experiment import CometExperiment
 from ..monkey_patching import check_module
 
 LOGGER = logging.getLogger(__name__)
@@ -26,17 +26,25 @@ class SHAPLogger(object):
         self.show = True
         self.title = title
 
-    def before(self, experiment, original, *args, **kwargs):
+    def before(
+        self, experiment: CometExperiment, original: Any, *args: Any, **kwargs: Any
+    ) -> Tuple[Any, Any]:
         """
         Little wrapper to make sure show is False
         """
         self.show = kwargs.get("show", True)
 
         kwargs["show"] = False
-        return (args, kwargs)
+        return args, kwargs
 
-    def after(self, experiment, original, results, *args, **kwargs):
-        # type: (BaseExperiment, Any, Any, Any, Any) -> None
+    def after(
+        self,
+        experiment: CometExperiment,
+        original: Any,
+        results: Any,
+        *args: Any,
+        **kwargs: Any
+    ) -> None:
         """
         Little wrapper to log figure, and show, if needed.
         """
