@@ -6,15 +6,15 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed without the express
-#  permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 
 
 import logging
 
+from ..messages import ParameterMessage
 from ..monkey_patching import check_module
 from .logger_utils import _check_callbacks_list, get_argument_bindings
 
@@ -39,7 +39,12 @@ optimizers_hyper_params = {
         "_l1_regularization_strength",
         "_l2_regularization_strength",
     ],
-    "RMSPropOptimizer": ["_learning_rate", "_decay", "_momentum", "_epsilon"],
+    "RMSPropOptimizer": [
+        "_learning_rate",
+        "_decay",
+        "_momentum",
+        "_epsilon",
+    ],
     "AdadeltaOptimizer": ["_lr", "_rho", "_epsilon"],
     "GradientDescentOptimizer": ["_learning_rate"],
     "MomentumOptimizer": ["_learning_rate", "_momentum", "_use_nesterov"],
@@ -79,7 +84,12 @@ def optimizer_logger(experiment, original, value, *args, **kwargs):
             if len(args) > 0:
                 LOGGER.debug("TENSORFLOW LOGGER CALLED")
                 params = extract_params_from_optimizer(args[0])
-                experiment._log_parameters(params, framework="tensorflow")
+                experiment._log_parameters(
+                    params,
+                    framework="tensorflow",
+                    source=ParameterMessage.source_autologger,
+                    flatten_nested=False,
+                )
 
         except Exception:
             LOGGER.error(
@@ -132,6 +142,57 @@ OPTIMIZER_V2_6 = [
     ("keras.optimizer_v2.rmsprop", "RMSprop.__init__"),
 ]
 
+# TF 2.9.0 have moved the optimizers once more
+OPTIMIZER_V2_9 = [
+    ("keras.optimizers.optimizer_v2.adam", "Adam.__init__"),
+    ("keras.optimizers.optimizer_v2.adadelta", "Adadelta.__init__"),
+    ("keras.optimizers.optimizer_v2.adagrad", "Adagrad.__init__"),
+    ("keras.optimizers.optimizer_v2.adamax", "Adamax.__init__"),
+    ("keras.optimizers.optimizer_v2.ftrl", "Ftrl.__init__"),
+    ("keras.optimizers.optimizer_v2.gradient_descent", "SGD.__init__"),
+    ("keras.optimizers.optimizer_v2.nadam", "Nadam.__init__"),
+    ("keras.optimizers.optimizer_v2.rmsprop", "RMSprop.__init__"),
+]
+
+# TF 2.11.0 have moved the optimizers once more
+OPTIMIZER_V2_11 = [
+    ("keras.optimizers.optimizer_experimental.adam", "Adam.__init__"),
+    ("keras.optimizers.optimizer_experimental.adadelta", "Adadelta.__init__"),
+    ("keras.optimizers.optimizer_experimental.adagrad", "Adagrad.__init__"),
+    ("keras.optimizers.optimizer_experimental.adamax", "Adamax.__init__"),
+    ("keras.optimizers.optimizer_experimental.ftrl", "Ftrl.__init__"),
+    ("keras.optimizers.optimizer_experimental.gradient_descent", "SGD.__init__"),
+    ("keras.optimizers.optimizer_experimental.nadam", "Nadam.__init__"),
+    ("keras.optimizers.optimizer_experimental.rmsprop", "RMSprop.__init__"),
+    ("keras.optimizers.optimizer_experimental.sgd", "SGD.__init__"),
+]
+
+# TF 2.12.0 have moved the optimizers once more
+OPTIMIZER_V2_12 = [
+    ("keras.optimizers.adam", "Adam.__init__"),
+    ("keras.optimizers.adadelta", "Adadelta.__init__"),
+    ("keras.optimizers.adagrad", "Adagrad.__init__"),
+    ("keras.optimizers.adamax", "Adamax.__init__"),
+    ("keras.optimizers.ftrl", "Ftrl.__init__"),
+    ("keras.optimizers.legacy.gradient_descent", "SGD.__init__"),
+    ("keras.optimizers.nadam", "Nadam.__init__"),
+    ("keras.optimizers.rmsprop", "RMSprop.__init__"),
+    ("keras.optimizers.sgd", "SGD.__init__"),
+]
+
+# TF 2.13.0 have moved the optimizers once more - its like never ending fun :)
+OPTIMIZER_V2_13 = [
+    ("keras.src.optimizers.adam", "Adam.__init__"),
+    ("keras.src.optimizers.adadelta", "Adadelta.__init__"),
+    ("keras.src.optimizers.adagrad", "Adagrad.__init__"),
+    ("keras.src.optimizers.adamax", "Adamax.__init__"),
+    ("keras.src.optimizers.ftrl", "Ftrl.__init__"),
+    ("keras.src.optimizers.legacy.gradient_descent", "SGD.__init__"),
+    ("keras.src.optimizers.nadam", "Nadam.__init__"),
+    ("keras.src.optimizers.rmsprop", "RMSprop.__init__"),
+    ("keras.src.optimizers.sgd", "SGD.__init__"),
+]
+
 ESTIMATOR_V1 = [
     ("tensorflow.python.estimator.canned.baseline", "BaselineClassifier.__init__"),
     ("tensorflow.python.estimator.canned.baseline", "BaselineRegressor.__init__"),
@@ -281,7 +342,12 @@ def estimator_logger(experiment, original, value, *args, **kwargs):
             # Turn all values into strings:
             for key in params:
                 params[key] = str(params[key])
-            experiment._log_parameters(params, framework="tensorflow")
+            experiment._log_parameters(
+                params,
+                framework="tensorflow",
+                source=ParameterMessage.source_autologger,
+                flatten_nested=False,
+            )
 
         except Exception:
             LOGGER.error(
@@ -305,7 +371,12 @@ def optimizer_logger_v2(experiment, original, value, *args, **kwargs):
                 name = config.pop("name")
                 params = {name + "_" + key: config[key] for key in config}
                 params["Optimizer"] = name
-                experiment._log_parameters(params, framework="tensorflow")
+                experiment._log_parameters(
+                    params,
+                    framework="tensorflow",
+                    source=ParameterMessage.source_autologger,
+                    flatten_nested=False,
+                )
 
         except Exception:
             LOGGER.error(
@@ -328,6 +399,22 @@ def patch(module_finder):
     for module, object_name in OPTIMIZER_V2_6:
         module_finder.register_after(module, object_name, optimizer_logger_v2)
 
+    # Register the v2.9.x methods
+    for module, object_name in OPTIMIZER_V2_9:
+        module_finder.register_after(module, object_name, optimizer_logger_v2)
+
+    # Register the v2.11.x methods
+    for module, object_name in OPTIMIZER_V2_11:
+        module_finder.register_after(module, object_name, optimizer_logger_v2)
+
+    # Register the v2.12.x methods
+    for module, object_name in OPTIMIZER_V2_12:
+        module_finder.register_after(module, object_name, optimizer_logger_v2)
+
+    # Register the v2.13.x methods
+    for module, object_name in OPTIMIZER_V2_13:
+        module_finder.register_after(module, object_name, optimizer_logger_v2)
+
     # Estimators:
     for module, object_name in ESTIMATOR_V2:
         module_finder.register_after(module, object_name, estimator_logger)
