@@ -6,32 +6,38 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed without the express
-#  permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 
 """
 Author: Boris Feld
 
-This module contains the main components of comet.ml client side
+This module contains the main components of comet client side
 
 """
+import logging
 import random
 from collections import OrderedDict
+from typing import Any, Dict, List, Optional
 
-from ._typing import Any, Dict, List, Optional
+from .logging_messages import (
+    METRICS_SAMPLER_MAX_OPERATION_STRING_VALUE_WARNING,
+    METRICS_SAMPLER_MIN_OPERATION_STRING_VALUE_WARNING,
+)
+
+LOGGER = logging.getLogger(__name__)
 
 
-class ReservoirSampler(object):
+class ReservoirSampler:
     def __init__(self, num_samples):
         """Create a new sampler with a certain reservoir size."""
         self.num_samples = num_samples
         self.num_items_seen = 0
-        self.reservoir = []  # type: List[float]
+        self.reservoir: List[Any] = []
 
-    def sample(self, item):
+    def sample(self, item: Any) -> None:
         """Sample an item and store in the reservoir if needed."""
         num_samples = self.num_samples
 
@@ -52,12 +58,10 @@ class ReservoirSampler(object):
 
 
 class BaseOperation(object):
-    def process(self, metric):
-        # type: (Dict[str, Any]) -> Optional[Dict[str, Any]]
+    def process(self, metric: Dict[str, Any]) -> Optional[Dict[str, Any]]:
         return metric
 
-    def get_saved_metric(self):
-        # type: () -> Optional[Dict[str, Any]]
+    def get_saved_metric(self) -> Optional[Dict[str, Any]]:
         return None
 
 
@@ -65,14 +69,14 @@ class FirstMessageOperation(BaseOperation):
     def __init__(self):
         self.first_message = None
 
-    def process(self, metric):
+    def process(self, metric: Dict[str, Any]) -> Optional[Dict[str, Any]]:
         if self.first_message is None:
             self.first_message = metric
             return None
 
         return metric
 
-    def get_saved_metric(self):
+    def get_saved_metric(self) -> Optional[Dict[str, Any]]:
         return self.first_message
 
 
@@ -80,12 +84,12 @@ class LastMessageOperation(BaseOperation):
     def __init__(self):
         self.previous = None
 
-    def process(self, metric):
+    def process(self, metric: Dict[str, Any]) -> Optional[Dict[str, Any]]:
         previous = self.previous
         self.previous = metric
         return previous
 
-    def get_saved_metric(self):
+    def get_saved_metric(self) -> Optional[Dict[str, Any]]:
         return self.previous
 
 
@@ -93,19 +97,28 @@ class MinMessageOperation(BaseOperation):
     def __init__(self):
         self.min_message = None
 
-    def process(self, metric):
+    def process(self, metric: Dict[str, Any]) -> Optional[Dict[str, Any]]:
+        value = metric["metric"]["metricValue"]
+        if not isinstance(value, (float, int)):
+            LOGGER.warning(
+                METRICS_SAMPLER_MIN_OPERATION_STRING_VALUE_WARNING,
+                value,
+                metric["metric"]["metricName"],
+            )
+            return metric
+
         if self.min_message is None:
             self.min_message = metric
-            return
+            return None
 
-        if metric["metric"]["metricValue"] < self.min_message["metric"]["metricValue"]:
+        if value < self.min_message["metric"]["metricValue"]:
             previous_min_message = self.min_message
             self.min_message = metric
             return previous_min_message
 
         return metric
 
-    def get_saved_metric(self):
+    def get_saved_metric(self) -> Optional[Dict[str, Any]]:
         return self.min_message
 
 
@@ -113,19 +126,28 @@ class MaxMessageOperation(BaseOperation):
     def __init__(self):
         self.max_message = None
 
-    def process(self, metric):
+    def process(self, metric: Dict[str, Any]) -> Optional[Dict[str, Any]]:
+        value = metric["metric"]["metricValue"]
+        if not isinstance(value, (float, int)):
+            LOGGER.warning(
+                METRICS_SAMPLER_MAX_OPERATION_STRING_VALUE_WARNING,
+                value,
+                metric["metric"]["metricName"],
+            )
+            return metric
+
         if self.max_message is None:
             self.max_message = metric
-            return
+            return None
 
-        if metric["metric"]["metricValue"] > self.max_message["metric"]["metricValue"]:
+        if value > self.max_message["metric"]["metricValue"]:
             previous_max_message = self.max_message
             self.max_message = metric
             return previous_max_message
 
         return metric
 
-    def get_saved_metric(self):
+    def get_saved_metric(self) -> Optional[Dict[str, Any]]:
         return self.max_message
 
 
@@ -140,19 +162,17 @@ class MetricsSampler(object):
             )
 
         self.sample_size = sample_size
-        self.samplers = OrderedDict()  # type: Dict[str, ReservoirSampler]
-        self.pipelines = {}  # type: Dict[str, List[BaseOperation]]
+        self.samplers: Dict[str, ReservoirSampler] = OrderedDict()
+        self.pipelines: Dict[str, List[BaseOperation]] = {}
         self.all_have_steps = True
 
-    def get_sampler(self, metric_name):
-        # type: (str) -> ReservoirSampler
+    def get_sampler(self, metric_name: str) -> ReservoirSampler:
         if metric_name not in self.samplers:
             self.samplers[metric_name] = ReservoirSampler(self.sample_size - 4)
 
         return self.samplers[metric_name]
 
-    def get_pipeline(self, metric_name):
-        # type: (str) -> List[BaseOperation]
+    def get_pipeline(self, metric_name: str) -> List[BaseOperation]:
         if metric_name not in self.pipelines:
             pipeline = [
                 FirstMessageOperation(),
@@ -164,8 +184,7 @@ class MetricsSampler(object):
 
         return self.pipelines[metric_name]
 
-    def sample_metric(self, metric):
-        # type: (Dict[str, Any]) -> None
+    def sample_metric(self, metric: Dict[str, Any]) -> None:
         """Sample the next element of the metric stream for the metric names.
         There are few rules about which messages we should keep at all costs:
         - The first message of the stream
@@ -198,8 +217,7 @@ class MetricsSampler(object):
 
         sampler.sample(metric)
 
-    def get_samples(self):
-        # type: () -> List[Dict[str, Any]]
+    def get_samples(self) -> List[Dict[str, Any]]:
         result = []
 
         for metric_name, sampler in self.samplers.items():
@@ -211,7 +229,7 @@ class MetricsSampler(object):
             for operation in pipeline:
                 saved_message = operation.get_saved_metric()
 
-                if saved_message:
+                if saved_message is not None:
                     result.append(saved_message)
 
         # Finally, sort everything back
