@@ -6,23 +6,18 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed without the express
-#  permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 
 """ The import hook which monkey patch modules
 """
 
 import functools
-import imp
 import inspect
 import logging
 import sys
-import types
-
-import six
 
 from .config import get_config, get_global_experiment
 from .logging_messages import COMET_DISABLED_AUTO_LOGGING_MSG
@@ -48,8 +43,7 @@ def is_module_already_imported(module):
 
 
 def _reset_already_imported_modules():
-    # Modify the set in place to be sure to keep the same reference every
-    # where
+    # Modify the set in place to be sure to keep the same reference everywhere
     ALREADY_IMPORTED_MODULES.clear()
 
 
@@ -85,7 +79,14 @@ class CustomFileLoader(object):
             # zipimporter doesn't use exec_module
             module = self.loader.load_module(self.fullname)
 
-        return self.finder._patch(module, self.fullname)
+        try:
+            return self.finder._patch(module, self.fullname)
+        except Exception:
+            LOGGER.debug(
+                "Failed to patch {}".format(self.fullname),
+                exc_info=True,
+            )
+            return module
 
     def create_module(self, spec):
         """Mandatory in Python 3.6 as we define the exec_module method"""
@@ -138,18 +139,6 @@ class CometModuleFinder(object):
         if self not in sys.meta_path:
             sys.meta_path.insert(0, self)
 
-    def find_module(self, fullname, path=None):
-        """Python 2 import hook"""
-        if fullname not in self.patcher_functions:
-            return
-
-        return self
-
-    def load_module(self, fullname):
-        """Python 2 import hook"""
-        module = self._get_module(fullname)
-        return self._patch(module, fullname)
-
     def find_spec(self, fullname, path=None, target=None):
         """Python 3 import hook"""
         if fullname not in self.patcher_functions:
@@ -165,36 +154,6 @@ class CometModuleFinder(object):
 
         return spec
 
-    def _get_module(self, fullname):
-        splitted_name = fullname.split(".")
-        parent = ".".join(splitted_name[:-1])
-
-        if fullname in sys.modules:
-            return sys.modules[fullname]
-
-        elif parent in sys.modules:
-            parent = sys.modules[parent]
-            module_path = imp.find_module(splitted_name[-1], parent.__path__)
-            return imp.load_module(fullname, *module_path)
-
-        else:
-            try:
-                module_path = imp.find_module(fullname)
-                return imp.load_module(fullname, *module_path)
-
-            except ImportError:
-                for p in sys.path:
-                    importer = get_pep_302_importer(p)
-
-                    # Ignore invalid paths
-                    if importer is None:
-                        continue
-
-                    module_path = importer.find_module(fullname)
-
-                    if module_path:
-                        return importer.load_module(fullname)
-
     def _patch(self, module, fullname):
         objects_to_patch = self.patcher_functions.get(fullname, {})
 
@@ -228,16 +187,8 @@ class CometModuleFinder(object):
 
         original_self = getattr(original, "__self__", None)
 
-        # Support classmethod
         if original_self and inspect.isclass(original_self):
             new_object = classmethod(new_object)
-            # Support staticmethod
-        elif (
-            six.PY2
-            and inspect.isclass(object_to_patch)
-            and isinstance(original, types.FunctionType)
-        ):
-            new_object = staticmethod(new_object)
 
         setattr(object_to_patch, object_path[-1], new_object)
 
@@ -260,6 +211,16 @@ def valid_new_args_kwargs(callback_return):
     return True
 
 
+class ExceptionCaughtSentinel:
+    def __init__(self, exception):
+        self.exception = exception
+
+    def __eq__(self, other):
+        if not isinstance(other, ExceptionCaughtSentinel):
+            return False
+        return self.exception is other.exception
+
+
 def Entrypoint(original, before=None, after=None, allow_empty_experiment=False):
     """The object that replaces the patched functions / methods.
 
@@ -309,11 +270,25 @@ def Entrypoint(original, before=None, after=None, allow_empty_experiment=False):
                         "Exception calling before callback %r", callback, exc_info=True
                     )
 
-        return_value = original(*args, **kwargs)
+        exception_raised = None
+        try:
+            return_value = original(*args, **kwargs)
+        except Exception as exception:
+            exception_raised = exception
+            return_value = ExceptionCaughtSentinel(exception)
 
         # Call after callbacks once we have the return value
         if should_run:
             for callback in after_callbacks:
+                callback_allows_exception = getattr(
+                    callback, "allow_after_exception", False
+                )
+                skip_callback = (
+                    exception_raised is not None and not callback_allows_exception
+                )
+                if skip_callback:
+                    continue
+
                 try:
                     new_return_value = callback(
                         experiment, original, return_value, *args, **kwargs
@@ -325,6 +300,9 @@ def Entrypoint(original, before=None, after=None, allow_empty_experiment=False):
                         "Exception calling after callback %r", callback, exc_info=True
                     )
 
+        if exception_raised is not None:
+            raise exception_raised
+
         return return_value
 
     # Simulate functools.wraps behavior but make it working with mocks
@@ -337,3 +315,7 @@ def Entrypoint(original, before=None, after=None, allow_empty_experiment=False):
     wrapper.__comet__ = True
 
     return wrapper
+
+
+def is_error_sentinel(value):
+    return isinstance(value, ExceptionCaughtSentinel)
