@@ -6,10 +6,9 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed
-#  without the express permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 
 import ast
@@ -17,12 +16,13 @@ import json
 import logging
 import os
 
+from . import logging_messages
 from ._typing import Any, Dict, Optional
 from .config import get_api_key, get_config
-from .connection import get_optimizer_api
+from .connection.connection_factory import get_optimizer_api
 from .exceptions import InvalidOptimizerParameters, OptimizerException
-from .experiment import BaseExperiment
-from .logging_messages import OPTIMIZER_KWARGS_CONSTRUCTOR_DEPRACETED
+from .experiment import CometExperiment
+from .messages import ParameterMessage
 
 LOGGER = logging.getLogger(__name__)
 
@@ -65,46 +65,50 @@ class Optimizer(object):
     listing them here.
 
     Args:
-        config: optional, if COMET_OPTIMIZER_ID is configured,
+        config (dict): If COMET_OPTIMIZER_ID is configured,
             otherwise is either a config dictionary, optimizer id,
             or a config filename.
-        trials: int (optional, default 1) number of trials
-            per parameter set to test.
-        verbose: boolean (optional, default 1) verbosity level
-            where 0 means no output, and 1 (or greater) means
+        trials (int): Number of trials per parameter set to test.
+        verbose (bool): Verbosity level where 0 means no output, and 1 (or greater) means
             to show more detail.
-        experiment_class: string or callable (optional, default
-            None), class to use (for example, OfflineExperiment).
+        experiment_class (str | callable): Class to use (for example, OfflineExperiment).
 
-    Examples:
+    Example:
+        ```python linenums="1"
+        import comet_ml
+
+        comet_ml.login()
 
-        ```python
         # Assume COMET_OPTIMIZER_ID is configured:
-        >>> opt = Optimizer()
+        opt = comet_ml.Optimizer()
 
         # An optimizer config dictionary:
-        >>> opt = Optimizer({"algorithm": "bayes", ...})
+        opt = comet_ml.Optimizer({"algorithm": "bayes", ...})
 
         # An optimizer id:
-        >>> opt = Optimizer("73745463463")
+        opt = comet_ml.Optimizer("73745463463")
 
         # A filename to a optimizer config file:
-        >>> opt = Optimizer("/tmp/mlhacker/optimizer.config")
+        opt = comet_ml.Optimizer("/tmp/mlhacker/optimizer.config")
         ```
 
-    To pass arguments to the `Experiment` constructor, pass them into the
-    `opt.get_experiments()` call, like so:
+        To pass arguments to the `Experiment` constructor, pass them into the
+        `opt.get_experiments()` call, like so:
 
-    ```python
-    >>> opt = Optimizer("/tmp/mlhacker/optimizer.config")
-    >>> for experiment in opt.get_experiments(
-    ...     project_name="my-project",
-    ...     auto_metric_logging=False,
-    ... ):
-    ...     loss = fit(model)
-    ...     experiment.log_metric("loss", loss")
-    ...     experiment.end()
-    ```
+        ```python linenums="1"
+        import comet_ml
+
+        comet_ml.login()
+        opt = comet_ml.Optimizer("/tmp/mlhacker/optimizer.config")
+
+        for experiment in opt.get_experiments(
+            project_name="my-project",
+            auto_metric_logging=False,
+        ):
+            loss = fit(model)
+            experiment.log_metric("loss", loss")
+            experiment.end()
+        ```
     """
 
     def __init__(
@@ -120,12 +124,11 @@ class Optimizer(object):
         The Optimizer constructor.
 
         Args:
-            config: (optional, if COMET_OPTIMIZER_ID is configured). Can
-                an optimizer config id, an optimizer config dictionary, or an
+            config (str | dict): Can an optimizer config id, an optimizer config dictionary, or an
                 optimizer config filename.
-            trials: int (optional, default 1) number of trials per parameter value set
-            verbose: int (optional, default 1) level of details to show; 0 means quiet
-            experiment_class: string (optional). Supported values are "Experiment" (the
+            trials (int): Number of trials per parameter value set
+            verbose (int): Level of details to show; 0 means quiet
+            experiment_class (str | callable): Supported values are "Experiment" (the
                 default) to use online Experiments or "OfflineExperiment" to use offline
                 Experiments. It can also be a callable (a function or a method) that returns an
                 instance of Experiment, OfflineExperiment, ExistingExperiment or
@@ -144,7 +147,7 @@ class Optimizer(object):
         self.experiment_class = experiment_class
 
         if kwargs:
-            LOGGER.warning(OPTIMIZER_KWARGS_CONSTRUCTOR_DEPRACETED)
+            LOGGER.warning(logging_messages.OPTIMIZER_KWARGS_CONSTRUCTOR_DEPRECATED)
 
         self.experiment_kwargs = kwargs
         self.config = get_config()
@@ -194,15 +197,25 @@ class Optimizer(object):
         (unless performing multiple trials per parameter set).
 
         Example:
+            ```python linenums="1"
+            import comet_ml
 
-            ```python
-            >>> for experiment in optimizer.get_experiments():
-            ...     loss = fit(x, y)
-            ...     experiment.log_metric("loss", loss)
+            comet_ml.login()
+            opt = comet_ml.Optimizer({"algorithm": "bayes", ...})
+
+            for experiment in opt.get_experiments():
+                loss = fit(x, y)
+                experiment.log_metric("loss", loss)
             ```
         """
         while True:
-            experiment = self.next(**kwargs)
+            experiment = None
+            try:
+                experiment = self.next(**kwargs)
+            except Exception:
+                LOGGER.error(
+                    logging_messages.OPTIMIZER_FAILED_TO_RETRIVE_DATA, exc_info=True
+                )
             if experiment is not None:
                 yield experiment
             else:
@@ -215,12 +228,16 @@ class Optimizer(object):
         once (unless performing multiple trials per parameter set).
 
         Example:
+            ```python linenums="1"
+            import comet_ml
 
-            ```python
-            >>> for parameters in optimizer.get_parameters():
-            ...     experiment = comet_ml.Experiment()
-            ...     loss = fit(x, y)
-            ...     experiment.log_metric("loss", loss)
+            comet_ml.login()
+            opt = comet_ml.Optimizer({"algorithm": "bayes", ...})
+
+            for parameters in optimizer.get_parameters():
+                experiment = comet_ml.start()
+                loss = fit(x, y)
+                experiment.log_metric("loss", loss)
             ```
         """
         while True:
@@ -230,8 +247,7 @@ class Optimizer(object):
             else:
                 break
 
-    def next(self, **kwargs):
-        # type: (Dict[str, str]) -> Optional[BaseExperiment]
+    def next(self, **kwargs: Dict[str, str]) -> Optional[CometExperiment]:
         """
         `Optimizer.next()` will return the next experiment for this
         sweep or search. All experiments will have a unique set of
@@ -242,16 +258,20 @@ class Optimizer(object):
         generator `Optimizer.get_experiments()`
 
         Args:
-            kwargs: (optional). Any keyword argument will be passed to the
+            kwargs (Any): Any keyword argument will be passed to the
                 Experiment class for creation. The API key is passed directly.
 
         Example:
+            ```python linenums="1"
+            import comet_ml
 
-            ```python
-            >>> experiment = optimizer.next()
-            >>> if experiment is not None:
-            ...     loss = fit(x, y)
-            ...     experiment.log_metric("loss", loss)
+            comet_ml.login()
+            opt = comet_ml.Optimizer({"algorithm": "bayes", ...})
+
+            experiment = optimizer.next()
+            if experiment is not None:
+                loss = fit(x, y)
+                experiment.log_metric("loss", loss)
             ```
         """
         data = self.next_data()
@@ -276,9 +296,12 @@ class Optimizer(object):
         `Optimizer.get_parameters()`
 
         Example:
+            ```python linenums="1"
+            import comet_ml
 
-            ```python
-            >>> experiment = optimizer.next_data()
+            comet_ml.login()
+            opt = comet_ml.Optimizer({"algorithm": "bayes", ...})
+            experiment = optimizer.next_data()
             ```
         """
         data = self._api.optimizer_next(self.id)
@@ -300,39 +323,56 @@ class Optimizer(object):
         Get the status from the optimizer server for this optimizer.
 
         Example:
+            Running the code sample:
+            ```python linenums="1"
+            import comet_ml
 
+            comet_ml.login()
+            opt = comet_ml.Optimizer({"algorithm": "bayes", ...})
+
+            opt.status()
             ```
-            >>> opt.status()
-            {'algorithm': 'grid',
-             'comboCount': 32,
-             'configSpaceSize': 10,
-             'endTime': None,
-             'id': 'c20b90ecad694e71bdb5702778eb8ac7',
-             'lastUpdateTime': None,
-             'maxCombo': 0,
-             'name': 'c20b90ecad694e71bdb5702778eb8ac7',
-             'parameters': {'x': {'max': 20,
-                                  'min': 0,
-                                  'scalingType': 'uniform',
-                                  'type': 'integer'}},
-             'retryCount': 0,
-             'spec': {'gridSize': 10,
-              'maxCombo': 0,
-              'metric': 'loss',
-              'minSampleSize': 100,
-              'randomize': False,
-              'retryLimit': 20,
-              'seed': 2997936454},
-             'startTime': 1558793292216,
-             'state': {
-              'sequence_i': 0,
-              'sequence_retry': 11,
-              'sequence_trial': 0,
-              'total': 32,
-             },
-             'status': 'running',
-             'trials': 1,
-             'version': '1.0.0'}
+
+            Will return the dict:
+            ```json
+            {
+                'algorithm': 'grid',
+                'comboCount': 32,
+                'configSpaceSize': 10,
+                'endTime': None,
+                'id': 'c20b90ecad694e71bdb5702778eb8ac7',
+                'lastUpdateTime': None,
+                'maxCombo': 0,
+                'name': 'c20b90ecad694e71bdb5702778eb8ac7',
+                'parameters': {
+                    'x': {
+                        'max': 20,
+                        'min': 0,
+                        'scalingType': 'uniform',
+                        'type': 'integer'
+                    }
+                },
+                'retryCount': 0,
+                'spec': {
+                    'gridSize': 10,
+                    'maxCombo': 0,
+                    'metric': 'loss',
+                    'minSampleSize': 100,
+                    'randomize': False,
+                    'retryLimit': 20,
+                    'seed': 2997936454
+                },
+                'startTime': 1558793292216,
+                'state': {
+                    'sequence_i': 0,
+                    'sequence_retry': 11,
+                    'sequence_trial': 0,
+                    'total': 32,
+                },
+                'status': 'running',
+                'trials': 1,
+                'version': '1.0.0'
+            }
             ```
         """
         results = self._api.optimizer_status(id=self.id)
@@ -340,16 +380,19 @@ class Optimizer(object):
 
     def get_id(self):
         """
-        Get the id of this optimizer, with Comet config variable.
+        Get the id of this optimizer.
 
         Example:
+            ```python linenums="1"
+            import comet_ml
 
-            ```
-            >>> opt.get_id()
-            COMET_OPTIMIZER_ID=87463746374647364
+            comet_ml.login()
+            opt = comet_ml.Optimizer({"algorithm": "bayes", ...})
+
+            opt.get_id()
             ```
         """
-        return "COMET_OPTIMIZER_ID=%s" % self.id
+        return str(self.id)
 
     def end(self, experiment):
         """
@@ -388,18 +431,19 @@ class Optimizer(object):
             )
         self._api.optimizer_update(self.id, pid, trial, result, score)
         # Log optimizer results with the experiment:
-        experiment.log_other("optimizer_metric", metric)
-        experiment.log_other("optimizer_metric_value", score)
-        experiment.log_other("optimizer_version", version)
-        experiment.log_other("optimizer_process", os.getpid())
-        experiment.log_other("optimizer_count", count)
-        experiment.log_other("optimizer_objective", objective)
-        experiment.log_other("optimizer_name", name)
-        experiment.log_other(
+        experiment._log_other("optimizer_metric", metric, include_context=False)
+        experiment._log_other("optimizer_metric_value", score, include_context=False)
+        experiment._log_other("optimizer_version", version, include_context=False)
+        experiment._log_other("optimizer_process", os.getpid(), include_context=False)
+        experiment._log_other("optimizer_count", count, include_context=False)
+        experiment._log_other("optimizer_objective", objective, include_context=False)
+        experiment._log_other("optimizer_name", name, include_context=False)
+        experiment._log_other(
             "optimizer_parameters",
             json.dumps(
                 {key: experiment.params.get(key, None) for key in status["parameters"]}
             ),
+            include_context=False,
         )
 
         return False  # don't force experiment to wait until uploading is done
@@ -559,8 +603,14 @@ class Optimizer(object):
                     return set(v1["values"]) != set(v2["values"])
         return v1 != v2
 
-    def _create_experiment(self, parameters, pid, trial, count, experiment_kwargs):
-        # type: (Any, Any, Any, Any, Dict[str, Any]) -> BaseExperiment
+    def _create_experiment(
+        self,
+        parameters: Dict[str, Any],
+        pid: Any,
+        trial: Optional[int],
+        count: Optional[int],
+        experiment_kwargs: Dict[str, Any],
+    ) -> CometExperiment:
         """
         Instantiates an Experiment, OfflineExperiment, or
         callable.
@@ -580,9 +630,9 @@ class Optimizer(object):
             experiment_kwargs["api_key"] = self.api_key
 
         if self.experiment_class == "Experiment":
-            exp = Experiment(**experiment_kwargs)  # type: BaseExperiment
+            exp: CometExperiment = Experiment(**experiment_kwargs)
         elif self.experiment_class == "OfflineExperiment":
-            exp = OfflineExperiment(**experiment_kwargs)  # type: BaseExperiment
+            exp: CometExperiment = OfflineExperiment(**experiment_kwargs)
         elif callable(self.experiment_class):
             exp = self.experiment_class(**experiment_kwargs)
         else:
@@ -592,9 +642,11 @@ class Optimizer(object):
 
         exp._set_optimizer(self, pid, trial, count)
 
-        exp.log_parameters(parameters)
+        exp._log_parameters(
+            parameters, source=ParameterMessage.source_autologger, flatten_nested=False
+        )
         # Log optimizer static information:
-        exp.log_other("optimizer_id", self.id)
-        exp.log_other("optimizer_pid", pid)
-        exp.log_other("optimizer_trial", trial)
+        exp._log_other("optimizer_id", self.id, include_context=False)
+        exp._log_other("optimizer_pid", pid, include_context=False)
+        exp._log_other("optimizer_trial", trial, include_context=False)
         return exp
