@@ -6,10 +6,9 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed
-#  without the express permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 
 import logging
@@ -24,5 +23,5 @@ LOGGER.warning(
     "You have imported comet_ml.papi; "
     + "this interface is deprecated. Please use "
     + "comet_ml.api instead. For more information, see: "
-    + "https://www.comet.ml/docs/python-sdk/releases/#release-300"
+    + "https://www.comet.com/docs/python-sdk/releases/#release-300"
 )
