@@ -6,10 +6,9 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed without the express
-#  permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 #
 """
@@ -24,8 +23,6 @@ import traceback
 
 from comet_ml.exceptions import BadCallbackArguments
 
-import six
-
 from ._typing import Any, Optional
 
 LOGGER = logging.getLogger(__name__)
@@ -60,35 +57,22 @@ def create_remote_call(call):
 
 def get_remote_action_definition(function):
 
-    if six.PY2:
-        argspec = inspect.getargspec(function)
+    argspec = inspect.getfullargspec(function)
 
-        arguments = argspec.args
+    # Check that the function accept a keyword argument named experiment
+    if (
+        "experiment" not in argspec.args
+        and argspec.varkw is None
+        and "experiment" not in argspec.kwonlyargs
+    ):
+        raise BadCallbackArguments(function)
 
-        # Check that the function accept a keyword argument named experiment
-        if "experiment" not in arguments and argspec.keywords is None:
-            raise BadCallbackArguments(function)
+    # It is forbidden to declare an argument name both as a positional and
+    # keyword-only argument, so we shouldn't get duplicates names
+    arguments = argspec.args + argspec.kwonlyargs
 
-        if "experiment" in arguments:
-            arguments.remove("experiment")
-
-    elif six.PY3:
-        argspec = inspect.getfullargspec(function)
-
-        # Check that the function accept a keyword argument named experiment
-        if (
-            "experiment" not in argspec.args
-            and argspec.varkw is None
-            and "experiment" not in argspec.kwonlyargs
-        ):
-            raise BadCallbackArguments(function)
-
-        # It is forbidden to declare an argument name both as a positional and
-        # keyword-only argument, so we shouldn't get duplicates names
-        arguments = argspec.args + argspec.kwonlyargs
-
-        if "experiment" in arguments:
-            arguments.remove("experiment")
+    if "experiment" in arguments:
+        arguments.remove("experiment")
 
     return {
         "functionName": function.__name__,
