@@ -7,10 +7,9 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed without
-#  the express permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 
 """
@@ -26,7 +25,7 @@ Examples:
     comet bootstrap_dir
 
     comet python script.py
-    comet python -p /usr/bin/python3.6 script.py
+    comet python -p /usr/bin/python3.9 script.py
 
     comet offline 60a1a617e4c24c8998cc78fa3bc7a31b.zip
     comet offline --csv 60a1a617e4c24c8998cc78fa3bc7a31b.zip
@@ -59,6 +58,7 @@ from comet_ml import __version__
 from . import (
     comet_check,
     comet_init,
+    comet_login,
     comet_models,
     comet_offline,
     comet_optimize,
@@ -120,6 +120,7 @@ def main(raw_args=sys.argv[1:]):
     add_subparser(subparsers, comet_python, "python")
     add_subparser(subparsers, comet_upload, "upload")
     add_subparser(subparsers, comet_init, "init")
+    add_subparser(subparsers, comet_login, "login")
 
     bootstrap_dir_parser = subparsers.add_parser(
         "bootstrap_dir",
