@@ -7,10 +7,9 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed without the express
-#  permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 
 import argparse
@@ -18,34 +17,30 @@ import importlib
 import logging
 import os
 import sys
+from urllib import request
 
 from comet_ml._typing import Any, List, Optional
 from comet_ml.config import (
+    COMET_URL_OVERRIDE_CONFIG_KEY,
     Config,
-    ConfigDictEnv,
-    ConfigEnvFileEnv,
-    ConfigIniEnv,
-    ConfigOSEnv,
+    get_check_tls_certificate,
+    get_comet_url_override,
     get_config,
-    get_ws_url,
-)
-from comet_ml.connection import (
-    _comet_version,
-    _debug_proxy_for_http,
-    _debug_proxy_for_ws,
-    get_and_format_proxy_for_ws,
-    get_http_session,
     get_optimizer_address,
+)
+from comet_ml.connection.connection_api import _debug_proxy_for_http
+from comet_ml.connection.connection_url_helpers import add_run_url
+from comet_ml.connection.http_session import get_comet_http_session
+from comet_ml.utils import (
+    get_comet_version,
     get_root_url,
-    get_run_id_url,
-    get_websocket_sslopt,
+    local_timestamp,
     sanitize_url,
     url_join,
 )
-from comet_ml.utils import local_timestamp, urlparse, urlunparse
 
-import six.moves.urllib_request as request
-import websocket
+from everett.ext.inifile import ConfigIniEnv
+from everett.manager import ConfigDictEnv, ConfigEnvFileEnv, ConfigOSEnv
 
 LOGGER = logging.getLogger("comet_ml")
 ADDITIONAL_ARGS = False
@@ -54,7 +49,6 @@ DELIMITERS_LENGTH = 80
 
 REQUESTS_CA_BUNDLE = "REQUESTS_CA_BUNDLE"
 CURL_CA_BUNDLE = "CURL_CA_BUNDLE"
-WEBSOCKET_CLIENT_CA_BUNDLE = "WEBSOCKET_CLIENT_CA_BUNDLE"
 
 
 def activate_debug():
@@ -68,83 +62,24 @@ def activate_debug():
             h.setLevel(logging.DEBUG)
 
     # Activate requests low-level request logging
-    try:
-        import http.client as http_client
-    except ImportError:
-        # Python 2
-        import httplib as http_client
+    import http.client as http_client
 
     http_client.HTTPConnection.debuglevel = 5
 
-    # Activate websocket low-level request logging
-    websocket.enableTrace(True)
-
-
-def check_ws_connection(websocket_url, debug, verify_tls):
-    # type: (str, bool, bool) -> bool
 
+def check_server_connection(server_address: str, verify_tls: bool) -> bool:
+    url = add_run_url(server_address)
     try:
-        (
-            http_proxy_host,
-            http_proxy_port,
-            http_proxy_auth,
-            proxy_type,
-        ) = get_and_format_proxy_for_ws(websocket_url)
-
-        sslopt = get_websocket_sslopt(verify_tls)
-
-        ws = websocket.create_connection(
-            websocket_url,
-            http_proxy_host=http_proxy_host,
-            http_proxy_port=http_proxy_port,
-            http_proxy_auth=http_proxy_auth,
-            proxy_type=proxy_type,
-            sslopt=sslopt,
-        )
-
-        if debug:
-            print("Sending test message")
-            print(ws.send("test"))
-
-            print("Receiving a message")
-            print(ws.recv())
-
-        ws.close()
-
-        return True
-    except Exception:
-        LOGGER.error("Error checking websocket connectivity", exc_info=True)
-        return False
-
-
-def get_default_ws_url(clientlib_address):
-    parsed_address = urlparse(clientlib_address)
-
-    if parsed_address.scheme == "http":
-        ws_scheme = "ws"
-    elif parsed_address.scheme == "https":
-        ws_scheme = "wss"
-    else:
-        raise NotImplementedError()
-
-    ws_host = parsed_address.netloc
-    ws_path = "ws/logger-ws"
-
-    return urlunparse((ws_scheme, ws_host, ws_path, None, None, None))
-
-
-def check_server_connection(server_address, verify_tls):
-    url = get_run_id_url(server_address)
-    try:
-        with get_http_session(verify_tls=verify_tls) as session:
+        api_key = "XXX"
+        with get_comet_http_session(verify_tls=verify_tls, api_key=api_key) as session:
             payload = {
-                "apiKey": "XXX",
+                "apiKey": api_key,
                 "local_timestamp": local_timestamp(),
                 "experimentKey": "YYY",
                 "offline": False,
                 "projectName": None,
                 "teamName": None,
-                "libVersion": _comet_version(),
+                "libVersion": get_comet_version(),
             }
             headers = {"Content-Type": "application/json;charset=utf-8"}
             session.post(url, data=payload, headers=headers)
@@ -154,10 +89,10 @@ def check_server_connection(server_address, verify_tls):
         return False
 
 
-def check_rest_api_connection(rest_api_url, verify_tls):
+def check_rest_api_connection(rest_api_url: str, verify_tls: bool) -> bool:
     url = url_join(rest_api_url, "account-details")
     try:
-        with get_http_session(verify_tls=verify_tls) as session:
+        with get_comet_http_session(verify_tls=verify_tls, api_key=None) as session:
             session.get(url)
             return True
     except Exception:
@@ -165,10 +100,10 @@ def check_rest_api_connection(rest_api_url, verify_tls):
         return False
 
 
-def check_optimizer_connection(optimizer_url, verify_tls):
+def check_optimizer_connection(optimizer_url: str, verify_tls: bool) -> bool:
     url = url_join(optimizer_url, "spec")
     try:
-        with get_http_session(verify_tls=verify_tls) as session:
+        with get_comet_http_session(verify_tls=verify_tls, api_key=None) as session:
             params = {"algorithmName": "bayes"}
             session.get(url, params=params)
             return True
@@ -177,8 +112,7 @@ def check_optimizer_connection(optimizer_url, verify_tls):
         return False
 
 
-def config_source(env):
-    # type: (Any) -> Optional[str]
+def config_source(env: Any) -> Optional[str]:
     if isinstance(env, ConfigOSEnv):
         return "environment variable"
     elif isinstance(env, ConfigEnvFileEnv):
@@ -192,13 +126,13 @@ def config_source(env):
         return None
 
 
-def check(args, rest=None):
+def check(args: Any) -> None:
     # Called via `comet upload EXP.zip`
     if args.debug:
         activate_debug()
 
     config = get_config()
-    verify_tls = config.get_bool(None, "comet.internal.check_tls_certificate")
+    verify_tls = get_check_tls_certificate(config)
 
     LOGGER.info("Comet Check")
     LOGGER.info("=" * DELIMITERS_LENGTH)
@@ -208,12 +142,13 @@ def check(args, rest=None):
     print("")
 
     # Clientlib
-    server_address = sanitize_url(config["comet.url_override"])
+    comet_url_override = get_comet_url_override(config)
+    server_address = sanitize_url(comet_url_override)
     server_address_config_origin = config_source(
-        config.get_config_origin("comet.url_override")
+        config.get_config_origin(COMET_URL_OVERRIDE_CONFIG_KEY)
     )
     LOGGER.info("Configured server address %r", server_address)
-    if server_address_config_origin:
+    if server_address_config_origin and comet_url_override is None:
         LOGGER.info("Server address was configured in %s", server_address_config_origin)
     else:
         LOGGER.info("Server address is the default one")
@@ -225,12 +160,14 @@ def check(args, rest=None):
     else:
         LOGGER.warning("Server connection is not ok")
 
+    print("")
+
     # Rest API
     LOGGER.info("=" * DELIMITERS_LENGTH)
     LOGGER.info("Checking connectivity to Rest API...")
     LOGGER.info("=" * DELIMITERS_LENGTH)
 
-    root_url = sanitize_url(get_root_url(config["comet.url_override"]))
+    root_url = sanitize_url(get_root_url(comet_url_override))
     rest_api_url = url_join(root_url, *["api/rest/", "v2" + "/"])
     LOGGER.info("Configured Rest API address %r", rest_api_url)
     if server_address_config_origin:
@@ -247,38 +184,7 @@ def check(args, rest=None):
     else:
         LOGGER.warning("REST API connection is not ok")
 
-    # Websocket
-    LOGGER.info("=" * DELIMITERS_LENGTH)
-    LOGGER.info("Checking connectivity to Websocket Server")
-    LOGGER.info("=" * DELIMITERS_LENGTH)
-
-    websocket_url = get_ws_url(None, config)
-    if websocket_url is None:
-        websocket_url = get_default_ws_url(server_address)
-        LOGGER.warning(
-            "No WS address configured on client side, fallback on default WS address %r, if that's incorrect set "
-            "the WS url through the `comet.ws_url_override` config key",
-            websocket_url,
-        )
-        websocket_url_config_origin = None
-    else:
-        websocket_url = websocket_url
-        websocket_url_config_origin = config_source(
-            config.get_config_origin("comet.ws_url_override")
-        )
-    LOGGER.info(
-        "Configured WS address %r",
-        websocket_url,
-    )
-    if websocket_url_config_origin:
-        LOGGER.info("WS address was configured in %s", websocket_url_config_origin)
     print("")
-    ws_connected = check_ws_connection(websocket_url, args.debug, verify_tls)
-    print("")
-    if ws_connected:
-        LOGGER.info("Websocket connection is ok")
-    else:
-        LOGGER.warning("Websocket connection is not ok")
 
     # Optimizer
     LOGGER.info("=" * DELIMITERS_LENGTH)
@@ -310,11 +216,9 @@ def check(args, rest=None):
     print("")
 
     log_system_info()
-    log_modules_info(["requests", "urllib3", "websocket"])
+    log_modules_info(["requests", "urllib3"])
     log_system_proxies_info()
-    log_proxies_info_for_urls(
-        http_urls=[server_address, rest_api_url], ws_urls=[websocket_url]
-    )
+    log_proxies_info_for_urls(http_urls=[server_address, rest_api_url])
     log_tls_verification(config)
     log_ca_bundles_paths()
 
@@ -325,11 +229,10 @@ def check(args, rest=None):
     LOGGER.info("-" * DELIMITERS_LENGTH)
     LOGGER.info("Server connectivity\t\t\t%s", server_connected)
     LOGGER.info("Rest API connectivity\t\t%r", rest_api_connected)
-    LOGGER.info("WS server connectivity\t\t%r", ws_connected)
     LOGGER.info("Optimizer server connectivity\t%r", optimizer_connected)
 
 
-def get_parser_arguments(parser):
+def get_parser_arguments(parser: argparse.ArgumentParser) -> None:
     parser.add_argument(
         "--debug",
         help="Activate the raw HTTP logs and be more verbose in general",
@@ -339,8 +242,7 @@ def get_parser_arguments(parser):
     )
 
 
-def log_system_info():
-    # type: () -> None
+def log_system_info() -> None:
     """Collects and logs the system information such as: Python version, platform, executable path"""
     LOGGER.info("=" * DELIMITERS_LENGTH)
     LOGGER.info("System Information")
@@ -348,8 +250,7 @@ def log_system_info():
     LOGGER.info("Python %s on %s\n%s", sys.version, sys.platform, sys.executable)
 
 
-def log_modules_info(mod_names):
-    # type: (List[str]) -> None
+def log_modules_info(mod_names: List[str]) -> None:
     """Collects and logs information about specific modules"""
     for mod_name in mod_names:
         try:
@@ -359,8 +260,7 @@ def log_modules_info(mod_names):
             LOGGER.error("Module %s not found" % mod_name, exc_info=True)
 
 
-def log_system_proxies_info():
-    # type: () -> None
+def log_system_proxies_info() -> None:
     """Collects and logs information about registered system proxies"""
     proxies = request.getproxies()
     if proxies:
@@ -376,8 +276,7 @@ def log_system_proxies_info():
     LOGGER.info("-" * DELIMITERS_LENGTH)
 
 
-def log_proxies_info_for_urls(http_urls, ws_urls):
-    # type: (List[str], List[str]) -> None
+def log_proxies_info_for_urls(http_urls: List[str]) -> None:
     # Log HTTP proxy info
     for url in http_urls:
         try:
@@ -386,16 +285,8 @@ def log_proxies_info_for_urls(http_urls, ws_urls):
         except Exception:
             LOGGER.error("Failed to check proxy for URL: %s", url, exc_info=True)
 
-    for ws_url in ws_urls:
-        try:
-            proxy = _debug_proxy_for_ws(ws_url)
-            log_proxy_info_for_url(ws_url, proxy)
-        except Exception:
-            LOGGER.error("Failed to check proxy for URL: %s", ws_url, exc_info=True)
 
-
-def log_proxy_info_for_url(url, proxy):
-    # type: (str, Optional[str]) -> None
+def log_proxy_info_for_url(url: str, proxy: Optional[str]) -> None:
     """Collects and logs information about proxies which proxy access to the specified list of URLs"""
     if proxy:
         LOGGER.info(
@@ -407,10 +298,9 @@ def log_proxy_info_for_url(url, proxy):
         LOGGER.info("%s -> NO PROXY", url)
 
 
-def log_tls_verification(config):
-    # type: (Config) -> None
+def log_tls_verification(config: Config) -> None:
     LOGGER.info("-" * DELIMITERS_LENGTH)
-    tls_verification = config.get_bool(None, "comet.internal.check_tls_certificate")
+    tls_verification = get_check_tls_certificate(config)
 
     if tls_verification:
         LOGGER.info("TLS Certificate verification is ENABLED")
@@ -418,7 +308,7 @@ def log_tls_verification(config):
         LOGGER.info("TLS Certification verification is DISABLED")
 
 
-def log_ca_bundles_paths():
+def log_ca_bundles_paths() -> None:
     LOGGER.info("=" * DELIMITERS_LENGTH)
     LOGGER.info("HTTP CA bundle paths")
     LOGGER.info("-" * DELIMITERS_LENGTH)
@@ -427,12 +317,12 @@ def log_ca_bundles_paths():
         LOGGER.info("%s: %s", CURL_CA_BUNDLE, os.environ.get(CURL_CA_BUNDLE))
     else:
         LOGGER.info("requests is using the default CA bundle")
+    try:
+        import certifi
 
-    if WEBSOCKET_CLIENT_CA_BUNDLE in os.environ:
-        LOGGER.info("=" * DELIMITERS_LENGTH)
-        LOGGER.info("WebSocket CA bundle paths")
-        LOGGER.info("-" * DELIMITERS_LENGTH)
-        LOGGER.info(os.environ[WEBSOCKET_CLIENT_CA_BUNDLE])
+        LOGGER.info(certifi.where())
+    except ImportError:
+        pass
 
 
 def main(args):
