@@ -7,10 +7,9 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed without
-#  the express permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 
 """
@@ -33,7 +32,7 @@ various ML libraries. It is called like:
 $ comet init
 
 The third option will allow you to set your onprem URL
-settings. For more information, see: https://www.comet.ml/docs/onprem/
+settings. For more information, see: https://www.comet.com/docs/onprem/
 
 $ comet init --onprem
 
@@ -47,7 +46,9 @@ import logging
 import os
 import sys
 
-from comet_ml.config import _init as init_api_key, get_config, init_onprem
+from comet_ml.config import get_config
+from comet_ml.config.config_init import _init as init_api_key, init_onprem
+from comet_ml.logging_messages import COMET_INIT_API_KEY_DEPRECATED
 
 LOGGER = logging.getLogger("comet_ml")
 ADDITIONAL_ARGS = True
@@ -114,6 +115,7 @@ def main(args):
 
 def init(parsed_args, remaining):
     if parsed_args.api_key:
+        LOGGER.warning(COMET_INIT_API_KEY_DEPRECATED)
         init_api_key(should_prompt_user=sys.stdin.isatty())
     elif parsed_args.onprem:
         # First, check, set URLs if necessary, and save them
