@@ -7,10 +7,9 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed without
-#  the express permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 
 """
@@ -49,6 +48,8 @@ import time
 import zipfile
 from collections import defaultdict
 
+from comet_ml.constants import DEFAULT_PROJECT_NAME
+
 from .._typing import Dict, Optional
 from ..config import (
     OFFLINE_EXPERIMENT_JSON_FILE_NAME,
@@ -58,10 +59,15 @@ from ..config import (
 from ..messages import (
     CloudDetailsMessage,
     FileNameMessage,
+    GitMetadataMessage,
+    InstalledPackagesMessage,
+    LogOtherMessage,
+    MetricMessage,
     ModelGraphMessage,
     OsPackagesMessage,
     ParameterMessage,
     RemoteAssetMessage,
+    StandardOutputMessage,
     SystemDetailsMessage,
     UploadFileMessage,
     WebSocketMessage,
@@ -166,10 +172,12 @@ class ArchivedExperiment(object):
         with self.archive.open(OFFLINE_EXPERIMENT_MESSAGES_JSON_FILE_NAME) as fp:
             for line in fp:
                 message = json.loads(line)
-                if (
-                    message["type"] == WebSocketMessage.type
-                    or message["type"] == ParameterMessage.type
-                ):
+                if message["type"] in [
+                    WebSocketMessage.type,
+                    ParameterMessage.type,
+                    MetricMessage.type,
+                    LogOtherMessage.type,
+                ]:
                     self.process_payload(message["payload"])
                 elif message["type"] == UploadFileMessage.type:
                     payload = message["payload"]
@@ -189,6 +197,10 @@ class ArchivedExperiment(object):
                     self.write(
                         "detail", UploadFileMessage.type, payload["upload_type"], size
                     )
+                elif message["type"] == StandardOutputMessage.type:
+                    payload = message["payload"]
+                    name = "stderr" if payload["stderr"] is True else "stdout"
+                    self.update_summary("data", name, payload["output"], True)
                 elif message["type"] in [
                     RemoteAssetMessage.type,
                     ModelGraphMessage.type,
@@ -196,6 +208,8 @@ class ArchivedExperiment(object):
                     SystemDetailsMessage.type,
                     CloudDetailsMessage.type,
                     FileNameMessage.type,
+                    InstalledPackagesMessage.type,
+                    GitMetadataMessage.type,
                 ]:
                     continue
                 else:
@@ -313,7 +327,7 @@ class ArchivedExperiment(object):
                             > self.config["comet.offline_sampling_size"]
                         ):
                             warning = (
-                                "--- THIS METRIC WILL BE DOWNSAMPLED TO %s ITEMS: see https://www.comet.ml/docs/python-sdk/warnings-errors/#offline-experiments"
+                                "--- THIS METRIC WILL BE DOWNSAMPLED TO %s ITEMS: see https://www.comet.com/docs/v2/api-and-sdk/python-sdk/warnings-errors/#offline-experiments"
                                 % self.config["comet.offline_sampling_size"]
                             )
                     if value is not None:
@@ -329,7 +343,7 @@ class ArchivedExperiment(object):
             line = fp.readline()
             experiment = json.loads(line)
             self.workspace = experiment["workspace"] or "DEFAULT"
-            self.project = experiment["project_name"] or "general"
+            self.project = experiment["project_name"] or DEFAULT_PROJECT_NAME
             self.experiment = experiment["offline_id"]
             for field, value in [
                 (
