@@ -7,10 +7,9 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed
-#  without the express permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 
 """
@@ -157,7 +156,7 @@ def optimize(parsed, subcommand_args):
     api_key = config["comet.api_key"]
     if api_key is None:
         raise Exception(
-            """Please set your API key: see https://www.comet.ml/docs/python-sdk/advanced/#python-configuration"""
+            """Please set your API key: see https://www.comet.com/docs/python-sdk/advanced/#python-configuration"""
         )
 
     if not (os.path.isfile(parsed.OPTIMIZER) or len(parsed.OPTIMIZER) == 32):
