@@ -6,43 +6,65 @@
 # | |__| (_) | | | | | |  __/ |_ _| | | | | | |
 #  \____\___/|_| |_| |_|\___|\__(_)_| |_| |_|_|
 #
-#  Sign up for free at http://www.comet.ml
+#  Sign up for free at https://www.comet.com
 #  Copyright (C) 2015-2021 Comet ML INC
-#  This file can not be copied and/or distributed without the express
-#  permission of Comet ML Inc.
+#  This source code is licensed under the MIT license.
 # *******************************************************
 
 from __future__ import print_function
 
-import atexit
 import calendar
 import getpass
+import io
 import json
 import logging
 import os
 import os.path
+import platform
 import random
-import re
+import string
 import subprocess
 import sys
 import tempfile
 import time
 import uuid
-from datetime import datetime
+import zipfile
+from datetime import datetime, timezone
+from typing import (
+    Any,
+    AnyStr,
+    Callable,
+    Dict,
+    List,
+    Optional,
+    Set,
+    TextIO,
+    Tuple,
+    Union,
+)
+
+from ._jupyter import _in_ipython_environment
+from .exceptions import StringValueTruncatedException
+
+if sys.version_info[:2] >= (3, 8):
+    import importlib.metadata as importlib_metadata
+else:
+    import importlib_metadata
+
+from urllib.parse import urlencode, urljoin, urlparse, urlsplit, urlunparse, urlunsplit
 
-import six
-from pkg_resources import DistributionNotFound, get_distribution
 from requests.models import PreparedRequest
-from six.moves.urllib.parse import urlparse, urlunparse
 
-from ._typing import IO, Any, Dict, Generator, List, Optional, Set, Tuple
-from .compat_utils import json_dump, subprocess_run
-from .json_encoder import NestedEncoder
-from .logging_messages import DATAFRAME_PROFILE_ERROR, MISSING_PANDAS_PROFILING
+from .json_encoder import NestedEncoder, NestedNumpyOrTensorEncoder
+from .logging_messages import (
+    CONVERT_UTILS_CONVERT_TO_STRING_TRUNCATED_WARNING,
+    MISSING_PANDAS_OR_YDATA_PROFILING,
+)
 
 LOGGER = logging.getLogger(__name__)
-LOG_ONCE_CACHE = set()  # type: Set[str]
+LOG_ONCE_CACHE: Set[str] = set()
 
+AWS_LAMBDA_ENVIRONMENT_MARKER_KEY = "LAMBDA_TASK_ROOT"
 
 if hasattr(time, "monotonic"):
     get_time_monotonic = time.monotonic
@@ -52,12 +74,14 @@ else:
     get_time_monotonic = time.time
 
 
-def log_once_at_level(logging_level, message, *args, **kwargs):
+def log_once_at_level(
+    logging_level: int, message: str, *args: Any, **kwargs: Any
+) -> None:
     """
     Log the given message once at the given level then at the DEBUG
     level on further calls.
 
-    This is a global log-once-per-session, as ooposed to the
+    This is a global log-once-per-session, as opposed to the
     log-once-per-experiment.
     """
     global LOG_ONCE_CACHE
@@ -69,8 +93,17 @@ def log_once_at_level(logging_level, message, *args, **kwargs):
         LOGGER.debug(message, *args, **kwargs)
 
 
-def merge_url(url, params):
-    # type: (str, Dict[Any, Any]) -> str
+def remove_log_once_at_level(message: str) -> None:
+    """
+    Clears the specified message from the global log-once-per-session cache, enabling it to be logged at the original
+    level again when ``log_once_at_level()`` is subsequently invoked.
+    """
+    global LOG_ONCE_CACHE
+
+    LOG_ONCE_CACHE.discard(message)
+
+
+def merge_url(url: str, params: Dict[Any, Any]) -> str:
     """
     Given an URL that might have query strings,
     combine with additional query strings.
@@ -86,41 +119,72 @@ def merge_url(url, params):
     return req.url
 
 
-def is_iterable(value):
+def url_join(base: str, *parts: Any, **kwargs: Any) -> str:
+    """Given a base and url parts (for example [workspace, project, id]) returns a full URL"""
+    # TODO: Enforce base to have a scheme and netloc?
+    result = base
+
+    for part in parts[:-1]:
+        if not part.endswith("/"):
+            raise ValueError("Intermediary part not ending with /")
+
+        result = urljoin(result, part)
+
+    result = urljoin(result, parts[-1])
+
+    if kwargs:
+        # merge the url with additional query args:
+        result = merge_url(result, kwargs)
+
+    return result
+
+
+def is_iterable(value: Any) -> bool:
     try:
         iter(value)
         return True
 
-    except TypeError:
+    except (TypeError, AttributeError):
         return False
 
 
-def is_list_like(value):
+def is_list_like(value: Any) -> bool:
     """Check if the value is a list-like"""
-    if is_iterable(value) and not isinstance(value, six.string_types):
+    if is_iterable(value) and not isinstance(value, str):
         return True
 
     else:
         return False
 
 
-def to_utf8(str_or_bytes):
+def to_utf8(str_or_bytes: AnyStr) -> AnyStr:
     if hasattr(str_or_bytes, "decode"):
         return str_or_bytes.decode("utf-8", errors="replace")
 
     return str_or_bytes
 
 
-def local_timestamp():
-    # type: () -> int
+def local_timestamp() -> int:
     """Return a timestamp in a format expected by the backend (milliseconds)"""
-    now = datetime.utcnow()
+    now = datetime.now(timezone.utc)
     timestamp_in_seconds = calendar.timegm(now.timetuple()) + (now.microsecond / 1e6)
-    timestamp_in_milliseconds = int(timestamp_in_seconds * 1000)
-    return timestamp_in_milliseconds
+    return timestamp_milliseconds(timestamp_in_seconds)
 
 
-def wait_for_done(check_function, timeout, progress_callback=None, sleep_time=1):
+def timestamp_milliseconds(timestamp_in_seconds: float) -> int:
+    return int(timestamp_in_seconds * 1000)
+
+
+def seconds_to_datetime_str(seconds: float) -> str:
+    return datetime.fromtimestamp(seconds).isoformat()
+
+
+def wait_for_done(
+    check_function: Callable,
+    timeout: float,
+    progress_callback: Optional[Callable] = None,
+    sleep_time: float = 1,
+) -> None:
     """Wait up to TIMEOUT seconds for the check function to return True"""
     end_time = time.time() + timeout
     while check_function() is False and time.time() < end_time:
@@ -134,8 +198,9 @@ def wait_for_done(check_function, timeout, progress_callback=None, sleep_time=1)
             time.sleep(sleep_time / 20.0)
 
 
-def read_unix_packages(package_status_file="/var/lib/dpkg/status"):
-    # type: (str) -> Optional[List[str]]
+def read_unix_packages(
+    package_status_file: str = "/var/lib/dpkg/status",
+) -> Optional[List[str]]:
     if os.path.isfile(package_status_file):
         package = None
         os_packages = []
@@ -157,47 +222,12 @@ def read_unix_packages(package_status_file="/var/lib/dpkg/status"):
         return None
 
 
-def write_file_like_to_tmp_file(file_like_object, tmpdir):
-    # type: (IO, str) -> str
-    # Copy of `shutil.copyfileobj` with binary / text detection
-
-    buf = file_like_object.read(1)
-
-    tmp_file = tempfile.NamedTemporaryFile(mode="w+b", dir=tmpdir, delete=False)
-
-    encode = False
-
-    # Detect binary/text
-    if not isinstance(buf, bytes):
-        encode = True
-        buf = buf.encode("utf-8")
-
-    tmp_file.write(buf)
-
-    # Main copy loop
-    while True:
-        buf = file_like_object.read(16 * 1024)
-
-        if not buf:
-            break
-
-        if encode:
-            buf = buf.encode("utf-8")
-
-        tmp_file.write(buf)
-
-    tmp_file.close()
-
-    return tmp_file.name
-
-
 ONE_KBYTE = float(1024)
 ONE_MBYTE = float(1024 * 1024)
 ONE_GBYTE = float(1024 * 1024 * 1024)
 
 
-def format_bytes(size):
-    # type: (float) -> str
+def format_bytes(size: float) -> str:
     """
     Given a size in bytes, return a sort string representation.
     """
@@ -211,20 +241,7 @@ def format_bytes(size):
         return "%d %s" % (size, "bytes")
 
 
-def get_file_extension(file_path):
-    if file_path is None:
-        return None
-
-    ext = os.path.splitext(file_path)[1]
-    if not ext:
-        return None
-
-    # Get rid of the leading "."
-    return ext[1::]
-
-
-def encode_metadata(metadata):
-    # type: (Optional[Dict[Any, Any]]) -> Optional[str]
+def encode_metadata(metadata: Optional[Dict[Any, Any]]) -> Optional[str]:
     if metadata is None:
         return None
 
@@ -236,23 +253,27 @@ def encode_metadata(metadata):
         return None
 
     try:
-        json_encoded = json.dumps(metadata, separators=(",", ":"), sort_keys=True)
+        json_encoded = json.dumps(
+            metadata, separators=(",", ":"), sort_keys=True, cls=NestedEncoder
+        )
         return json_encoded
     except Exception:
         LOGGER.info("invalid metadata, expecting JSON-encodable object", exc_info=True)
         return None
 
 
-def get_comet_version():
-    # type: () -> str
+def encode_asset_data_to_json(asset_data: Any) -> str:
+    return json.dumps(asset_data, separators=(",", ":"), cls=NestedNumpyOrTensorEncoder)
+
+
+def get_comet_version() -> str:
     try:
-        return get_distribution("comet_ml").version
-    except DistributionNotFound:
+        return importlib_metadata.version("comet_ml")
+    except importlib_metadata.PackageNotFoundError:
         return "Please install comet with `pip install comet_ml`"
 
 
-def get_user():
-    # type: () -> str
+def get_user() -> str:
     try:
         return getpass.getuser()
     except KeyError:
@@ -265,70 +286,9 @@ def get_user():
         return "unknown"
 
 
-def log_asset_folder(folder, recursive=False, extension_filter=None):
-    # type: (str, bool, Optional[List[str]]) -> Generator[Tuple[str, str], None, None]
-    extension_filter_set = None
-
-    if extension_filter is not None:
-        extension_filter_set = set(extension_filter)
-
-    if recursive:
-        for dirpath, _, file_names in os.walk(folder):
-            for file_name in file_names:
-
-                if extension_filter_set:
-                    file_extension = os.path.splitext(file_name)[1]
-
-                    if file_extension not in extension_filter_set:
-                        continue
-
-                file_path = os.path.join(dirpath, file_name)
-                yield (file_name, file_path)
-    else:
-        file_names = sorted(os.listdir(folder))
-        for file_name in file_names:
-            file_path = os.path.join(folder, file_name)
-            if os.path.isfile(file_path):
-
-                if extension_filter_set:
-                    file_extension = os.path.splitext(file_name)[1]
-                    if file_extension not in extension_filter_set:
-                        continue
-
-                yield (file_name, file_path)
-
-
-def parse_version_number(raw_version_number):
-    # type: (str) -> Tuple[int, int, int]
-    """
-    Parse a valid "INT.INT.INT" string, or raise an
-    Exception. Exceptions are handled by caller and
-    mean invalid version number.
-    """
-    converted_version_number = [int(part) for part in raw_version_number.split(".")]
-
-    if len(converted_version_number) != 3:
-        raise ValueError(
-            "Invalid version number %r, parsed as %r",
-            raw_version_number,
-            converted_version_number,
-        )
-
-    # Make mypy happy
-    version_number = (
-        converted_version_number[0],
-        converted_version_number[1],
-        converted_version_number[2],
-    )
-    return version_number
-
-
-def format_version_number(version_number):
-    # type: (Tuple[int, int, int]) -> str
-    return ".".join(map(str, version_number))
-
-
-def valid_ui_tabs(tab=None, preferred=False):
+def valid_ui_tabs(
+    tab: Optional[str] = None, preferred: bool = False
+) -> Union[str, List[str]]:
     """
     List of valid UI tabs in browser.
     """
@@ -380,7 +340,7 @@ def valid_ui_tabs(tab=None, preferred=False):
         raise ValueError("invalid tab name; tab should be in %r" % preferred_names)
 
 
-def shape(data):
+def shape(data: Any) -> List[int]:
     """
     Given a nested list or a numpy array,
     return the shape.
@@ -395,7 +355,7 @@ def shape(data):
             return []
 
 
-def tensor_length(data):
+def tensor_length(data: Any) -> int:
     """
     Get the length of a tensor/list.
     """
@@ -409,71 +369,31 @@ def tensor_length(data):
     return length
 
 
-def makedirs(name, exist_ok=False):
-    """
-    Replacement for Python2's version lacking exist_ok
-    """
-    if not os.path.exists(name) or not exist_ok:
-        os.makedirs(name)
+def truncate_to_length_or_raise(
+    value: Optional[str],
+    length: Optional[int],
+    raise_on_truncate: bool = False,
+) -> (str, bool):
+    out_value, truncated = truncate_to_length(value, length=length)
+    if truncated and raise_on_truncate:
+        raise StringValueTruncatedException(
+            CONVERT_UTILS_CONVERT_TO_STRING_TRUNCATED_WARNING % (length, value)
+        )
+
+    return out_value, truncated
 
 
-def clean_and_check_root_relative_path(root, relative_path):
-    # type: (str, str) -> str
-    """
-    Given a root and a relative path, resolve the relative path to get an
-    absolute path and make sure the resolved path is a child to root. Cases
-    where it could not be the case would be if the `relative_path` contains `..`
-    or if one part of the relative path is a symlink going above the root.
+def truncate_to_length(value: Optional[str], length: Optional[int]) -> (str, bool):
+    if value is None or length is None:
+        return value, False
 
-    Return the absolute resolved path and raises a ValueError if the root path
-    is not absolute or if the resolved relative path goes above the root.
-    """
-    if not os.path.isabs(root):
-        raise ValueError("Root parameter %r should an absolute path" % root)
+    if len(value) > length:
+        return value[:length], True
 
-    if not root.endswith(os.sep):
-        root = root + os.sep
-
-    real_root = os.path.realpath(root)
-
-    joined_path = os.path.join(real_root, relative_path)
-    resolved_path = os.path.realpath(joined_path)
-
-    if not resolved_path.startswith(real_root):
-        raise ValueError("Final path %r is outside of %r" % (resolved_path, real_root))
-
-    return resolved_path
+    return value, False
 
 
-def check_if_path_relative_to_root(root, absolute_path):
-    # type: (str, str) -> bool
-    if not os.path.isabs(root):
-        raise ValueError("Root parameter %r should an absolute path" % root)
-
-    root_full_path = os.path.realpath(root) + os.sep
-    full_path = os.path.realpath(absolute_path)
-
-    return full_path.startswith(root_full_path)
-
-
-def verify_data_structure(datatype, data):
-    # Raise an error if anything wrong
-    if datatype == "curve":
-        if (
-            ("x" not in data)
-            or ("y" not in data)
-            or ("name" not in data)
-            or (not isinstance(data["name"], str))
-            or (len(data["x"]) != len(data["y"]))
-        ):
-            raise ValueError(
-                "'curve' requires lists 'x' and 'y' of equal lengths, and string 'name'"
-            )
-    else:
-        raise ValueError("invalid datatype %r: datatype must be 'curve'" % datatype)
-
-
-def proper_registry_model_name(name):
+def proper_registry_model_name(name: str) -> str:
     """
     A proper registry model name is:
         * lowercase
@@ -492,73 +412,71 @@ def proper_registry_model_name(name):
     return name
 
 
-def is_interactive():
+def is_interactive() -> bool:
     """
     Returns True if in interactive mode
     """
     return bool(getattr(sys, "ps1", sys.flags.interactive))
 
 
-def safe_filename(filename):
-    """
-    Given a value, turn it into a valid filename.
-
-    1. Remove the spaces
-    2. Replace anything not alpha, '-', '_', or '.' with '_'
-    3. Remove duplicate '_'
-    """
-    string = str(filename).strip().replace(" ", "_")
-    string = re.sub(r"(?u)[^-\w.]", "_", string)
-    return re.sub(r"_+", "_", string)
-
-
-def get_dataframe_profile_html(dataframe, minimal):
-    # type: (Any, Optional[bool]) -> Optional[str]
+def get_dataframe_profile_html(
+    dataframe: Any, minimal: Optional[bool]
+) -> Optional[str]:
     """
     Log a pandas dataframe profile.
     """
-    try:
-        import pandas_profiling
+    dataframe_profiling = _import_dataframe_profiling()
 
+    if dataframe_profiling is None:
+        return None
+
+    if minimal:  # earlier versions of pandas_profiling did not support minimal
+        profile = dataframe_profiling.ProfileReport(dataframe, minimal=minimal)
+    else:
+        profile = dataframe_profiling.ProfileReport(dataframe)
+    html = profile.to_html()  # type: str
+    html = html.replace("http:", "https:")
+    return html
+
+
+def _import_dataframe_profiling():
+    """
+    Pandas_profiling was deprecated by its creators in favor
+    of the ydata_profiling. Comet still supports usage of the old package.
+    """
+    try:
+        import ydata_profiling
+
+        return ydata_profiling
     except ImportError:
-        LOGGER.warning(MISSING_PANDAS_PROFILING)
-        return None
-    except Exception:
-        LOGGER.warning(MISSING_PANDAS_PROFILING, exc_info=True)
-        return None
+        try:
+            import pandas_profiling
 
-    try:
-        if minimal:  # earlier versions of pandas_profiling did not support minimal
-            profile = pandas_profiling.ProfileReport(dataframe, minimal=minimal)
-        else:
-            profile = pandas_profiling.ProfileReport(dataframe)
-        html = profile.to_html()  # type: str
-        html = html.replace("http:", "https:")
-        return html
-    except Exception:
-        LOGGER.warning(DATAFRAME_PROFILE_ERROR, exc_info=True)
-
-        return None
+            return pandas_profiling
+        except ImportError:
+            LOGGER.warning(MISSING_PANDAS_OR_YDATA_PROFILING, exc_info=True)
+            return None
 
 
-def clean_string(string):
-    if string:
-        return "".join([char for char in string if char not in ["'", '"', " "]])
+def clean_string(val: str) -> str:
+    if val:
+        return "".join([char for char in val if char not in ["'", '"', " "]])
     else:
         return ""
 
 
-def get_api_key_from_user():
+def get_api_key_from_user() -> str:
     """
-    Get the Comet API key fromthe user.
+    Get the Comet API key from the user.
     """
-    from .config import get_config
+    from .config import get_comet_url_override, get_config
 
-    client_url = get_config("comet.url_override")
+    settings = get_config()
+    client_url = get_comet_url_override(settings)
     root_url = sanitize_url(get_root_url(client_url))
 
     print(
-        "Please enter your Comet API key from {root_url}api/my/settings/".format(
+        "Please paste your Comet API key from {root_url}api/my/settings/".format(
             root_url=root_url
         )
     )
@@ -567,8 +485,7 @@ def get_api_key_from_user():
     return api_key
 
 
-def parse_remote_uri(uri):
-    # type: (Optional[str]) -> Optional[str]
+def parse_remote_uri(uri: Optional[str]) -> Optional[str]:
     if not uri:
         return None
 
@@ -584,13 +501,6 @@ def parse_remote_uri(uri):
         return None
 
 
-def make_template_filename(group=None):
-    if group is None:
-        group = random.randint(0, 10000000)
-
-    return "template_projector_config-%s.json" % group
-
-
 class ImmutableDict(dict):
     def _immutable(self, *args, **kws):
         raise TypeError("object is immutable")
@@ -626,38 +536,16 @@ class IterationProgressCallback(object):
                     LOGGER.debug("Error calling the progress callback", exc_info=True)
 
 
-def atexit_unregister(func):
-    # type: (Any) -> None
-    if six.PY2:
-        to_delete = []
-
-        # First iterate to detect exithandlers to remove
-        for exithandlers in atexit._exithandlers:
-            if exithandlers[0] == func:
-                to_delete.append(exithandlers)
-
-        # Then remove them but we cannot assume that the list hasn't changed, so uses remove which is inefficient
-        for exithandler_to_delete in to_delete:
-            try:
-                atexit._exithandlers.remove(exithandler_to_delete)
-            except ValueError:
-                pass
-    else:
-        return atexit.unregister(func)
-
-
-def generate_guid():
-    # type: () -> str
+def generate_guid() -> str:
     """Generate a GUID"""
     return uuid.uuid4().hex
 
 
-def compact_json_dump(data, fp):
-    return json_dump(data, fp, sort_keys=True, separators=(",", ":"), cls=NestedEncoder)
+def compact_json_dump(data: Any, fp: TextIO) -> None:
+    json.dump(data, fp, sort_keys=True, separators=(",", ":"), cls=NestedEncoder)
 
 
-def get_root_url(url):
-    # type: (str) -> str
+def get_root_url(url: str) -> str:
     """Remove the path, params, query and fragment from a given URL"""
     parts = urlparse(url)
     scheme, netloc, path, params, query, fragment = parts
@@ -665,10 +553,9 @@ def get_root_url(url):
     return urlunparse((scheme, netloc, "", "", "", ""))
 
 
-def sanitize_url(url, ending_slash=True):
-    # type: (str, Optional[bool]) -> str
-    """Sanitize an URL, checking that it is a valid URL and ensure it contains an ending slash / or
-    that it doesn't contains an ending slash depending on the value of ending_slash"""
+def sanitize_url(url: str, ending_slash: bool = True) -> str:
+    """Sanitize URL, checking that it is a valid URL and ensure it contains an ending slash / or
+    that it doesn't contain an ending slash depending on the value of ending_slash"""
     parts = urlparse(url)
     scheme, netloc, path, params, query, fragment = parts
 
@@ -687,8 +574,7 @@ def sanitize_url(url, ending_slash=True):
     return urlunparse((scheme, netloc, path, params, query, fragment))
 
 
-def metric_name(name, prefix=None):
-    # type: (str, Optional[str]) -> str
+def metric_name(name: str, prefix: Optional[str] = None) -> str:
     if prefix:
         return "%s.%s" % (prefix, name)
     else:
@@ -696,9 +582,11 @@ def metric_name(name, prefix=None):
 
 
 def subprocess_run_and_check(
-    args, timeout, exception_warning_log_message, return_code_warning_log_message
-):
-    # type: (List[str], int, str, str) -> Optional[bytes]
+    args: List[str],
+    timeout: int,
+    exception_warning_log_message: str,
+    return_code_warning_log_message: str,
+) -> Optional[bytes]:
     """Run the command passed as a List of string with the given timeout (ignored on Python 2.7).
 
     If the command ran but it's exit code is not 0, show the return_code_warning_log_message as a
@@ -708,7 +596,13 @@ def subprocess_run_and_check(
     None.
     """
     try:
-        completed_process = subprocess_run(args, timeout=timeout)
+        completed_process = subprocess.run(
+            args,
+            check=False,
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
+            timeout=timeout,
+        )
     except Exception:
         LOGGER.warning(exception_warning_log_message, exc_info=True)
         return None
@@ -717,11 +611,125 @@ def subprocess_run_and_check(
         completed_process.check_returncode()
     except subprocess.CalledProcessError as exception:
         LOGGER.warning(return_code_warning_log_message, exc_info=True)
-        LOGGER.debug("Conda env export stdout: %r", exception.output)
-        LOGGER.debug("Conda env export stderr: %r", exception.stderr)
+        message = "%s\n" % exception
+        if exception.output is not None and len(exception.output) > 0:
+            message += to_utf8(exception.output) + "\n"
+        if exception.stderr is not None and len(exception.stderr) > 0:
+            message += to_utf8(exception.stderr)
+
+        LOGGER.warning(message)
         return None
     except Exception:
         LOGGER.warning(exception_warning_log_message, exc_info=True)
         return None
 
     return completed_process.stdout
+
+
+def tag_experiment_with_commit_number(experiment):
+    if sys.version_info.major == 3:
+        commit = subprocess.check_output(
+            "git rev-parse HEAD", shell=True, universal_newlines=True
+        ).strip()
+    else:
+        commit = subprocess.check_output("git rev-parse HEAD", shell=True).strip()
+
+    commit = commit[:8]
+    experiment.add_tag(commit)
+
+
+def tag_experiment_with_transport_layer_identifier(experiment):
+    """Adds to the experiment TAG with transport layer identifier taken from environment"""
+    experiment.add_tag("HTTP: %r" % True)
+
+
+def is_aws_lambda_environment() -> bool:
+    """Allows to check if we are executing in AWS lambda environment"""
+    return AWS_LAMBDA_ENVIRONMENT_MARKER_KEY in os.environ
+
+
+def find_logger_spec(logger_spec: str) -> str:
+    if is_aws_lambda_environment():
+        return "simple"
+
+    if logger_spec == "default":
+        if _in_ipython_environment():
+            logger_spec = "simple"
+        # MacOS default start mode is now spawn and fork is unsafe https://bugs.python.org/issue33725
+        elif platform.system() == "Darwin":
+            logger_spec = "simple"
+        else:
+            logger_spec = "native"
+    else:
+        logger_spec = logger_spec or None  # in case of ""
+
+    return logger_spec
+
+
+def optional_update(destination: Dict[str, Any], source: Dict[str, Any]) -> None:
+    """
+    the equivalent of destination.update(source),
+    except that if a (key, value) pair in source is None,
+    then this key is skipped.
+
+    this essentially automates the pattern:
+    if variable is not None:
+        destination["variable"] = variable
+    """
+    for key, value in source.items():
+        if value is not None:
+            destination[key] = value
+
+
+def read_git_patch_zip(patch_data: bytes) -> bytes:
+    zip_patch = io.BytesIO(patch_data)
+    archive = zipfile.ZipFile(zip_patch)
+    return archive.read("git_diff.patch")
+
+
+def compress_git_patch(git_patch: Any) -> Tuple[zipfile.ZipFile, str]:
+    # Create a zip
+    zip_dir = tempfile.mkdtemp()
+
+    zip_path = os.path.join(zip_dir, "patch.zip")
+    archive = zipfile.ZipFile(zip_path, "w", zipfile.ZIP_DEFLATED)
+    archive.writestr("git_diff.patch", git_patch)
+    archive.close()
+
+    return archive, zip_path
+
+
+def random_ascii_string(length: int = 6) -> str:
+    return "".join(random.choice(string.ascii_letters) for _ in range(length))
+
+
+def masked_api_key(api_key: str) -> Optional[str]:
+    if api_key is None:
+        return None
+
+    length = len(api_key)
+    if length < 5:
+        return api_key
+
+    prefix = api_key[:5]
+    return prefix + "*" * (length - 5)
+
+
+def is_valid_experiment_key(experiment_key: str) -> bool:
+    """Validate an experiment_key; returns True or False"""
+    return (
+        isinstance(experiment_key, str)
+        and experiment_key.isalnum()
+        and (32 <= len(experiment_key) <= 50)
+    )
+
+
+def create_asset_url(prefix, **query_arguments):
+    if prefix is None:
+        return None
+
+    splitted = list(urlsplit(prefix))
+
+    splitted[3] = urlencode(query_arguments)
+
+    return urlunsplit(splitted)
